@@ -83,6 +83,8 @@ letmulti VARN*:names[]               @stmt
 @expr pcall   METHODN:method expr:base pcall.. expr*:args[] @stmt
 @expr mt      MT:mt expr:target expr:option @stmt  @cmpl
 @expr op      expr:first expr:second expr*:args[] @stmt  @cmpl
+@expr invoke  expr:func expr*args[] @stmt @cmpl
+@expr proceed METHODN:method expr:base expr*args[]
 
 @expr tri     expr:bool expr:tvalue expr:fvalue @stmt  @cmpl
 @expr alt     expr*:bools[]  @stmt   @cmpl
