@@ -26,8 +26,8 @@ SWAP  sfi sfi
 
 PINIo sfi OBJ
 
-SETEBP sfi
-CHECKEBP sfi
+SETESP sfi
+CHECKESP sfi
 RET
 RETn sfi sfi
 RETa sfi sfi
@@ -292,7 +292,7 @@ def write_KCODE_asm_c(f, kc):
 		knh_Object_RCinc(op->a%d);''' % c)
 		c += 1
 	f.write('''
-		DP(o)->prev_op = (knh_codejmp_t*)op;
+		DP(o)->prev_op = (knh_kode_t*)op;
 	}
 	return (knh_code_t*)op;
 }
@@ -316,9 +316,9 @@ def write_label(f):
 	CPROTO.append('void knh_Asm_remapAddress(Ctx *ctx, Asm *o)')
 	f.write('''
 
-typedef void (*frewrite)(Ctx *ctx, Asm *o, knh_code_t *pc_start, knh_codejmp_t *op);
+typedef void (*frewrite)(Ctx *ctx, Asm *o, knh_code_t *pc_start, knh_kode_t *op);
 
-void knh_rewrite_addr(Ctx *ctx, Asm *o, knh_code_t *pc_start, knh_codejmp_t *op)
+void knh_rewrite_addr(Ctx *ctx, Asm *o, knh_code_t *pc_start, knh_kode_t *op)
 {
 	int offset = DP(o)->labels[op->a1].offset;
 	if(offset == -1) {
@@ -332,7 +332,7 @@ void knh_rewrite_addr(Ctx *ctx, Asm *o, knh_code_t *pc_start, knh_codejmp_t *op)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_rewrite_NOP(Ctx *ctx, Asm *o, knh_code_t *pc_start, knh_codejmp_t *op)
+void knh_rewrite_NOP(Ctx *ctx, Asm *o, knh_code_t *pc_start, knh_kode_t *op)
 {
 	//DBG2_P("%p, op=%d", op, op->opcode);
 }
@@ -358,7 +358,7 @@ void knh_Asm_writeAddress(Ctx *ctx, Asm *o, knh_code_t *pc_start)
 	while(1) {
 		int opcode = KNH_OPCODE(pc);
 		if(opcode == OPCODE_HALT) return ; 
-		OPREWRITE[opcode](ctx, o, pc_start, (knh_codejmp_t*)pc);
+		OPREWRITE[opcode](ctx, o, pc_start, (knh_kode_t*)pc);
 		pc += OPSIZE[opcode];
 	}
 }
@@ -423,18 +423,20 @@ def write_dump(f):
 	f.write('''
 #define knh_write__addr(ctx, w, a)  knh_printf(ctx, w, "%p", a)
 #define knh_write__boolean(ctx, w, a)  knh_write__s(ctx, w, (a) ? "true" : "false")
-#define knh_write__cid(ctx, w, a) knh_write__class(ctx, w, a)
+#define knh_write__cid(ctx, w, a) knh_write_cid(ctx, w, a)
 #define knh_write__float(ctx, w, a) knh_write__f(ctx, w, a)
 #define knh_write__integer(ctx, w, a) knh_write_integerfmt(ctx, w, KNH_INTEGER_FMT, a)
 #define knh_write__OBJ(ctx, w, a)  knh_format(ctx, w, METHODN__k, a, KNH_NULL)
 #define knh_write__sfi(ctx, w, a)  knh_printf(ctx, w, "sfp[%d]", (int)a);
 #define knh_write__sfx(ctx, w, a)  knh_printf(ctx, w, "sfp[%d]+%d", (int)a.i, (int)a.n)
 #define knh_write__ushort(ctx, w, a)    knh_write__i(ctx, w, (knh_int_t)a)
+#define knh_write__mn(ctx, w, a)    knh_write_mn(ctx, w, (knh_methodn_t)a)
+#define knh_write__type(ctx, w, a)    knh_write_type(ctx, w, (knh_type_t)a)
 
 static
 void KNH_DUMP_OPCODE(Ctx *ctx, knh_code_t *pc, OutputStream *w, Method *mtd, char *name)
 {
-	knh_codejmp_t *op = (knh_codejmp_t*)pc;
+	knh_kode_t *op = (knh_kode_t*)pc;
 	int line = 0;
 	if(IS_Method(mtd)) {
 		line = knh_Method_pctoline(mtd, pc);
@@ -518,7 +520,7 @@ void knh_code_thread(Ctx *ctx, knh_code_t *pc, void **codeaddr)
 {
 #ifdef KNH_USING_THREADEDCODE
 	while(1) {
-		knh_codejmp_t *op = (knh_codejmp_t*)pc;
+		knh_kode_t *op = (knh_kode_t*)pc;
 		KNH_ASSERT_OPCODE(op->opcode);
 		//DBG2_P("%p opcode=%d next=%p", op, op->opcode, op->nextaddr);
 		op->codeaddr = codeaddr[op->opcode];
@@ -532,7 +534,7 @@ void knh_code_thread(Ctx *ctx, knh_code_t *pc, void **codeaddr)
 		case %s : ''' % kc.OPCODE)
 	f.write('''
 			DBG2_P("%p GOTO %p", op, op->jumppc);
-			op->jumpaddr = codeaddr[((knh_codejmp_t*)(op->jumppc))->opcode];
+			op->jumpaddr = codeaddr[((knh_kode_t*)(op->jumppc))->opcode];
 			break;
 		}
 		pc += OPSIZE[op->opcode];
@@ -549,10 +551,10 @@ METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
 {
 	static void **globalMachineToJump = NULL;
 	//register knh_code_t *pc = DP((KLRCode*)DP(sfp[-1].mtd)->code)->code;
-	register knh_code_t *pc = ctx->ebp[0].pc;
+	register knh_code_t *pc = sfp[-1].pc;
 	DBG2_ASSERT(pc == DP((KLRCode*)DP(sfp[-1].mtd)->code)->code);
 	L_HEAD:;
-	switch(((knh_codejmp_t*)pc)->opcode) {
+	switch(((knh_kode_t*)pc)->opcode) {
 ''');
 	for kc in KCODE_LIST:
 		LC = ''
@@ -569,7 +571,7 @@ METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
 ''' % (kc.OPCODE, LC, kc.ctype, kc.ctype, getmacro(kc, 'L_HEAD'), kc.SIZE))
 	f.write('''
 	}
-	KNH_WARNING(ctx, "unknown opcode=%d", ((knh_codejmp_t*)pc)->opcode);
+	KNH_WARNING(ctx, "unknown opcode=%d", ((knh_kode_t*)pc)->opcode);
 	goto L_HEAD;
 }
 #endif/*KNH_USING_THREADEDCODE*/
@@ -587,7 +589,7 @@ METHOD knh_KLRCode_exec(Ctx *ctx, knh_sfp_t *sfp)
 	f.write('''
 	};
 	
-	register knh_code_t *pc = ctx->ebp[0].pc;
+	register knh_code_t *pc = sfp[-1].pc;
 	DBG2_ASSERT(pc == DP((KLRCode*)DP(sfp[-1].mtd)->code)->code);
 	//register knh_code_t *pc =  DP((KLRCode*)DP(sfp[-1].mtd)->code)->code;
 	goto *OPJUMP[KNH_OPCODE(pc)]; /* this is needed to init */
@@ -747,10 +749,10 @@ def write_cfunc(f):
 	write_chapter(f, "CODE_GENERATOR")
 	f.write('''
 
-typedef int (*fhaslabel)(knh_codejmp_t *op, knh_code_t *pc_start);
+typedef int (*fhaslabel)(knh_kode_t *op, knh_code_t *pc_start);
 
 static
-int knh_hasLabel_LABEL(knh_codejmp_t *op, knh_code_t *jumppc)
+int knh_hasLabel_LABEL(knh_kode_t *op, knh_code_t *jumppc)
 {
 	return (op->jumppc == jumppc);
 }
@@ -758,7 +760,7 @@ int knh_hasLabel_LABEL(knh_codejmp_t *op, knh_code_t *jumppc)
 /* ------------------------------------------------------------------------ */
 
 static
-int knh_hasLabel_NOP(knh_codejmp_t *op, knh_code_t *jumppc)
+int knh_hasLabel_NOP(knh_kode_t *op, knh_code_t *jumppc)
 {
 	return 0;
 }
@@ -784,7 +786,7 @@ int knh_hasLabel(knh_code_t *pc_start, knh_code_t *jumppc)
 	int res = 0;
 	knh_code_t *pc = pc_start;
 	while(res == 0) {
-		knh_codejmp_t *op = (knh_codejmp_t*)pc;
+		knh_kode_t *op = (knh_kode_t*)pc;
 		if(op->opcode == OPCODE_HALT) return 0; 
 		res = OPHASLABEL[op->opcode](op, jumppc);
 	}
@@ -817,7 +819,7 @@ void KNH_KLR_LABEL(Ctx *ctx, OutputStream *w, knh_code_t *pc_start, knh_code_t *
 	int step = 1;
 	knh_println(ctx, w, STEXT("{"));
 	while(1) {
-		knh_codejmp_t *op = (knh_codejmp_t*)pc;
+		knh_kode_t *op = (knh_kode_t*)pc;
 		switch(op->opcode) {
 		case OPCODE_HALT:
 			knh_println(ctx, w, STEXT("}"));
