@@ -72,6 +72,9 @@ class Source :
 		self.add_fieldn('default')
 		self.add_fieldn('typeof')
 		self.add_fieldn('defined')
+		self.add_fieldn('super')
+		self.add_fieldn('proceed')
+		self.add_fieldn('delegate')
 		self.add_fieldn('copy')
 		self.add_fieldn('format')
 		self.add_fieldn('GLOBAL')  ## GLOBALINDEX
@@ -523,7 +526,11 @@ def parse_flag(anno, oplist):
 def read_source_cblock(fpath, zone, text, data):
 	if text.find('@data') > 0 : return
 	functype = parse_functype(text)
-	if functype == None or functype.find('static') >= 0: return
+	if functype == None or functype.find('static') >= 0: 
+		if functype != None and functype.find('_fdefault') > 0 :
+			funcname = parse_funcname(functype)
+			data.FUNC_MAP[funcname] = functype
+		return
 	
 	if(data.lastfpath != fpath):
 		data.lastfpath = fpath;
