@@ -195,13 +195,37 @@ typedef struct knh_Range_t {
 
 /* ------------------------------------------------------------------------ */
 /* @class Array Object knh_Array_t @Cyclic @Param1 */
+/* @flag Array.Dim ARY:1 (%s)->h.flag 'is:set:*:*' */
 
 typedef struct knh_Array_t {
 	knh_hObject_t h;
 	struct knh_Object_t **list;
 	size_t size;
+	size_t capacity;
 } knh_Array_t;
 
+/* ------------------------------------------------------------------------ */
+/* @class IArray Object knh_IArray_t @Cyclic @Param1 */
+/* @flag IArray.Dim ARY:1 (%s)->h.flag 'is:set:*:*' */
+
+typedef struct knh_IArray_t {
+	knh_hObject_t h;
+	knh_integer_t *ilist;
+	size_t size;
+	size_t capacity;
+} knh_IArray_t;
+
+/* ------------------------------------------------------------------------ */
+/* @class FArray Object knh_FArray_t @Cyclic @Param1 */
+/* @flag FArray.Dim ARY:1 (%s)->h.flag 'is:set:*:*' */
+
+typedef struct knh_FArray_t {
+	knh_hObject_t h;
+	knh_float_t *flist;
+	size_t size;
+	size_t capacity;
+} knh_FArray_t;
+
 /* ------------------------------------------------------------------------ */
 /* @class Iterator Object knh_Iterator_struct @Param1 */
 
@@ -466,18 +490,14 @@ typedef struct knh_Method {
 /* @flag Mapper.MapMap MMF:1 (%s)->h.flag              'is:set:*:*' */
 
 typedef struct knh_Mapper {
-	knh_flag_t   flag;
-	knh_ushort_t size;
-	knh_class_t  scid;
-	knh_class_t  tcid;
-	union {
-		knh_fmapper     fmap;
-		struct knh_Mapper_t *m1;
-	};
+	knh_flag_t   flag; knh_ushort_t size;
+	knh_class_t  scid; knh_class_t  tcid;
+	//knh_fmapper     fmap;
 	union {
 		Object*      mapdata;
-		struct knh_Mapper_t *m2;
+		struct knh_Mapper_t *m1;
 	};
+	struct knh_Mapper_t *m2;
 } knh_Mapper_struct;
 
 /* ------------------------------------------------------------------------ */
@@ -593,7 +613,7 @@ typedef struct {
 
 typedef struct knh_BytesConv {
 	struct knh_String_t *name;
-	f_bconv fbconv;
+	knh_fbcnv fbconv;
 #ifdef KNH_USING_ICONV
 	iconv_t iconv_d;
 #endif
@@ -610,7 +630,7 @@ typedef struct knh_StringUnit {
 	knh_hClassSpec_struct spec;
 	knh_fvcabnew fnew;
 	knh_fvcabcmp fcmp;
-	f_bconv     fbconv;
+	knh_fbcnv     fbconv;
 	struct knh_BytesConv_t *bconv;
 	knh_ushort_t bytelen;
 	knh_ushort_t charlen;
@@ -744,6 +764,7 @@ typedef struct knh_Exception {
 
 typedef struct knh_ExceptionHandler {
 	jmp_buf jmpbuf;
+	knh_sfp_t *esp;
 	knh_code_t *pc;
 	struct knh_Exception_t* caught;
 } knh_ExceptionHandler_struct;
