@@ -107,6 +107,8 @@ typedef struct knh_tokens_t {
 #define knh_Stmt_setMAPNEXT(s,b)   knh_Stmt_setMemo1(s,b)
 #define knh_Stmt_isVOID(s)         knh_Stmt_isMemo1(s)
 #define knh_Stmt_setVOID(s,b)      knh_Stmt_setMemo1(s,b)
+#define knh_Stmt_isTHISSUPER(s)    knh_Stmt_isMemo1(s)
+#define knh_Stmt_setTHISSUPER(s,b) knh_Stmt_setMemo1(s,b)
 
 /* ------------------------------------------------------------------------ */
 
@@ -116,10 +118,6 @@ typedef struct knh_tokens_t {
 #define KNH_LVALUE      1
 
 typedef struct knh_Stmt {
-//	knh_fileid_t fileid;
-//	knh_sline_t   line;
-//	knh_flag_t   flag;
-//	knh_stmt_t   stt;
 	knh_type_t   type;
 	knh_short_t   line_end;
 	knh_ushort_t size;
@@ -169,9 +167,9 @@ typedef struct {
 		knh_code_t *jumppc;
 	};
 	void *nextaddr;
-} knh_codejmp_t;
+} knh_kode_t;
 
-#define KNH_OPCODE(pc)          ((knh_codejmp_t*)pc)->opcode
+#define KNH_OPCODE(pc)          ((knh_kode_t*)pc)->opcode
 
 #define KNH_ASSERT_OPCODE(c) \
 	if(c > OPCODE_NOP) {\
@@ -180,8 +178,10 @@ typedef struct {
 	}\
 
 /* ------------------------------------------------------------------------ */
-/* @class Asm Object knh_Asm_struct */
-/* @flag Asm.Cancelled CPF DP(%s)->flag 'is:set:*:*' */
+/* @class Asm Object knh_Asm_struct @Private */
+/* @flag Asm.Cancelled  CPF DP(%s)->flag 'is:set:*:*' */
+/* @flag Asm.PROCEED    CPF DP(%s)->flag 'is:set:*:*' */
+/* @flag Asm.YEILD      CPF DP(%s)->flag 'is:set:*:*' */
 
 #ifndef KONOHA_LOCALSIZE
 #define KONOHA_LOCALSIZE 128
@@ -229,7 +229,7 @@ typedef struct knh_Asm {
 	knh_sline_t            line;
 	struct knh_Bytes_t* elf;
 	struct knh_Bytes_t* dwarf;
-	knh_codejmp_t         *prev_op;
+	knh_kode_t         *prev_op;
 } knh_Asm_struct;
 
 
