@@ -55,7 +55,7 @@ extern "C" {
 #define CLASS_unknown        ((knh_class_t)-2)
 #define knh_Class_cid(c)     (knh_class_t)(c)->cid
 #define KNH_FLAG_CF2OF(f)        (f)
-#define knh_tClass_isPMZ(cid)    (ctx->tClass[cid].p1 != CLASS_Nue)
+#define knh_Class_isGenerics(cid)    (ctx->tClass[cid].p1 != CLASS_Nue)
 #define KNH_FLAG_SF_FIELD          KNH_FLAG_T1
 #define STRUCT_ISFIELD(sid)        ((sid & KNH_FLAG_SF_FIELD) == KNH_FLAG_SF_FIELD)
 #define BSIZE_TOSID(bsize)         (((knh_struct_t)bsize)|KNH_FLAG_SF_FIELD)
@@ -67,6 +67,7 @@ extern "C" {
 #define CLASSN(cid)   knh_tClass_CLASSN(ctx, cid)
 #define CLASSNo(o)    knh_tClass_CLASSN(ctx, knh_Object_cid(o))
 #define KNH_DEF(ctx, cid)  konoha_getClassDefaultValue(ctx, cid)
+#define KNH_DEF(ctx, cid)  konoha_getClassDefaultValue(ctx, cid)
 #define KNH_FLAG_TF_NN                KNH_FLAG_T0
 #define TYPE_ISNULLABLE(t)            ((t & KNH_FLAG_TF_NN)==0)
 #define IS_NNTYPE(t)                  ((t & KNH_FLAG_TF_NN)==KNH_FLAG_TF_NN)
@@ -98,7 +99,6 @@ extern "C" {
 	}\
 
 
-#define KNH_SETJUMP(hdlr)   setjmp(DP((ExceptionHandler*)hdlr)->jmpbuf)
 #define knh_Float_value(o)    (o)->value
 #define LRUMap                          HashMap
 #define knh_LRUMap                      knh_HashMap_struct
@@ -136,7 +136,7 @@ extern "C" {
 #define knh_write__x(ctx, w, n)   knh_write__ifmt(ctx, w, KNH_INT_FMTX, n)
 #define knh_write__f(ctx, w, f)  knh_write__ffmt(ctx, w, KNH_FLOAT_FMT, f)
 #define knh_write__e(ctx, w, f)  knh_write__ffmt(ctx, w, KNH_FLOAT_FMTE, f)
-#define knh_write__fn(ctx, w, fn)   knh_write__s(ctx, w, FIELDN(fn))
+#define knh_write_fn(ctx, w, fn)   knh_write__s(ctx, w, FIELDN(fn))
 #define knh_write__O(ctx, w, o)    knh_format(ctx, w, METHODN__k, o, KNH_NULL)
 #define knh_String_value(o)   ((o)->str)
 #define knh_String_tochar(o)  (char*)((o)->str)
@@ -166,33 +166,6 @@ extern "C" {
 #define knh_tMethodField_size               knh_Array_size(DP(ctx->sys)->tMethodFields)
 #define knh_Class_getMethod(ctx, c, mn)    knh_Class_getMethod__(ctx, c, mn, 0)
 #define knh_Class_findMethod(ctx, c, mn)   knh_Class_getMethod__(ctx, c, mn, 1)
-#define FIELDN_NONAME    ((knh_fieldn_t)-1)
-#define FIELDN_NEWID     ((knh_fieldn_t)-2)
-#define FIELDN_return    FIELDN_
-#define METHODN_NONAME   ((knh_methodn_t)-1)
-#define METHODN_NEWID    ((knh_methodn_t)-2)
-#define KNH_FLAG_FN_U1         KNH_FLAG_T0
-#define KNH_FLAG_FN_U2         KNH_FLAG_T1
-#define KNH_FLAG_FN_SUPER      (KNH_FLAG_T0|KNH_FLAG_T1)
-#define FIELDN_IS_SUPER(fnq)       ((fnq & KNH_FLAG_FN_SUPER) == KNH_FLAG_FN_SUPER)
-#define FIELDN_IS_U1(fnq)          ((fnq & KNH_FLAG_FN_U1) == KNH_FLAG_FN_U1)
-#define FIELDN_IS_U2(fnq)          ((fnq & KNH_FLAG_FN_U2) == KNH_FLAG_FN_U2)
-#define FIELDN_UNMASK(fnq)         (fnq & (~(KNH_FLAG_FN_SUPER|KNH_FLAG_FN_U1|KNH_FLAG_FN_U2)))
-#define KNH_FLAG_MN_SUPER        KNH_FLAG_T0
-#define KNH_FLAG_MN_GETTER       KNH_FLAG_T1
-#define KNH_FLAG_MN_SETTER       KNH_FLAG_T2
-#define KNH_FLAG_MN_MOVTEXT      (KNH_FLAG_T1|KNH_FLAG_T2)
-#define KNH_FLAG_MN_FIELDN       (~(KNH_FLAG_T0|KNH_FLAG_T1|KNH_FLAG_T2))
-#define METHODN_IS_GETTER(mn)   ((mn & KNH_FLAG_MN_MOVTEXT) == KNH_FLAG_MN_GETTER)
-#define METHODN_TO_GETTER(mn)   (mn | KNH_FLAG_MN_GETTER)
-#define METHODN_IS_SETTER(mn)   ((mn & KNH_FLAG_MN_MOVTEXT) == KNH_FLAG_MN_SETTER)
-#define METHODN_TO_SETTER(mn)   (mn | KNH_FLAG_MN_SETTER)
-#define METHODN_IS_MOVTEXT(mn)  ((mn & KNH_FLAG_MN_MOVTEXT) == KNH_FLAG_MN_MOVTEXT)
-#define METHODN_TO_MOVTEXT(mn)  (mn | KNH_FLAG_MN_MOVTEXT)
-#define METHODN_TOFIELDN(mn)     (mn & KNH_FLAG_MN_FIELDN)
-#define FIELDN(fn) knh_String_tochar(knh_tfieldn_name(ctx, fn))
-#define FILEIDN(fileid) knh_String_tochar(knh_tfileid_name(ctx, fileid))
-#define IS_SWEEP(f)   (f == knh_Object_RCsweep)
 #define KNH_FLAG_PF_STDERR      KNH_FLAG_T0
 #define KNH_FLAG_PF_EOL         KNH_FLAG_T1
 #define KNH_FLAG_PF_TIME        KNH_FLAG_T2
@@ -205,22 +178,23 @@ extern "C" {
 
 
 #define KNH_THROWf(ctx, fmt, ...) \
-	char throwbuf_[256]; \
-	knh_snprintf(throwbuf_, sizeof(throwbuf_), fmt, ## __VA_ARGS__); \
-	knh_throwException(ctx, new_Exception__b(ctx, B(throwbuf_)), KNH_SAFEFILE(__FILE__), __LINE__); \
+	char throcwb_[256]; \
+	knh_snprintf(throcwb_, sizeof(throcwb_), fmt, ## __VA_ARGS__); \
+	knh_throwException(ctx, new_Exception__b(ctx, B(throcwb_)), KNH_SAFEFILE(__FILE__), __LINE__); \
 
 
 #define TODO_THROW(ctx) knh_throw_TODO(ctx, (char*)__FILE__, __LINE__, (char*)__FUNCTION__)
+#define FIELDN(fn) knh_String_tochar(knh_tfieldn_name(ctx, fn))
+#define FILEIDN(fileid) knh_String_tochar(knh_tfileid_name(ctx, fileid))
 
 /* ======================================================================== */
 /* [PROTOTYPE] */
-KNHAPI(String*) new_String_bconv(Ctx *ctx, String *s, f_bconv bconv);
 KNHAPI(void) konoha_addAffineMapper(Ctx *ctx, knh_class_t scid, char *text, knh_float_t scale, knh_float_t shift);
 KNHAPI(knh_Array_t*) new_Array(Ctx *ctx, knh_class_t p1, size_t capacity);
 KNHAPI(void) knh_Array_add(Ctx *ctx, Array *o, Any *value);
 KNHAPI(Bytes*) new_Bytes(Ctx *ctx, size_t capacity);
 KNHAPI(knh_bytes_t) knh_Bytes_tobytes(Bytes *o);
-KNHAPI(void) knh_Bytes_clear(Bytes *o);
+KNHAPI(void) knh_Bytes_clear(Bytes *o, size_t pos);
 KNHAPI(void) knh_Bytes_putc(Ctx *ctx, Bytes *o, int ch);
 KNHAPI(void) knh_Bytes_write(Ctx *ctx, Bytes *o, knh_bytes_t v);
 KNHAPI(knh_bytes_t) new_bytes(char *c);
@@ -240,6 +214,7 @@ KNHAPI(char*) knh_format_bytes(char *buf, size_t bufsiz, knh_bytes_t t);
 KNHAPI(knh_int_t) knh_bytes_toint(knh_bytes_t t);
 KNHAPI(knh_float_t) knh_bytes_tofloat(knh_bytes_t t);
 KNHAPI(knh_int64_t) knh_bytes_toint64(knh_bytes_t t);
+KNHAPI(String*) new_String__fbcnv(Ctx *ctx, String *s, knh_fbcnv fbcnv, BytesConv *bc);
 KNHAPI(Exception*) new_Exception(Ctx *ctx, String *msg);
 KNHAPI(Exception*) new_Exception__b(Ctx *ctx, knh_bytes_t msg);
 KNHAPI(void) knh_perrno(Ctx *ctx, char *emsg, char *func, char *file, int line);
@@ -289,8 +264,6 @@ KNHAPI(void) konoha_shell(konoha_t konoha);
 KNHAPI(char*) knh_format_ospath(Ctx *ctx, char *buf, size_t bufsiz, knh_bytes_t path);
 KNHAPI(void) konoha_SETv(Ctx *ctx, Object **v, Object *o);
 KNHAPI(void) konoha_FINALv(Ctx *ctx, Object **v);
-KNHAPI(knh_ftraverse) konoha_sweep();
-KNHAPI(int) konoha_issweep(knh_ftraverse ftr);
 KNHAPI_(Object*) knh_boxing(Ctx *ctx, knh_sfp_t *sfp, knh_class_t cid);
 KNHAPI(knh_class_t) konoha_addClass(Ctx *ctx, knh_flag_t flag, char *name, int method_size);
 KNHAPI(knh_class_t) konoha_addVirtualClass(Ctx *ctx, char *name, int method_size);
@@ -312,6 +285,7 @@ KNHAPI(void) knh_sfp_format(Ctx *ctx, knh_sfp_t *sfp, knh_methodn_t mn, OutputSt
 KNHAPI(void) knh_throw_TODO(Ctx *ctx, char *file, int line, char *func);
 KNHAPI(char*) konoha_doing(Ctx *ctx, char *msg);
 KNHAPI(void) knh_throwException(Ctx *ctx, Exception *e, char *file, int line);
+KNHAPI(void) knh_throw__s(Ctx *ctx, char *msg, char *file, int line);
 KNHAPI(void) knh_throw(Ctx *ctx, Object *e, char *file, int line);
 KNHAPI(konoha_t) konoha_open(size_t ctxsize, size_t stacksize);
 KNHAPI(void) konoha_close(konoha_t konoha);
