@@ -42,18 +42,12 @@ extern "C" {
 /* ../src/api/arrayapi.c */
 METHOD knh__Array_new(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_new__array(Ctx *ctx, knh_sfp_t *sfp);
-METHOD knh__Array_new__array2D(Ctx *ctx, knh_sfp_t *sfp);
-METHOD knh__Array_new__array3D(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_new__init(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_add(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opLshift(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opSize(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_get(Ctx *ctx, knh_sfp_t *sfp);
-METHOD knh__Array_get2D(Ctx *ctx, knh_sfp_t *sfp);
-METHOD knh__Array_get3D(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_set(Ctx *ctx, knh_sfp_t *sfp);
-METHOD knh__Array_set2D(Ctx *ctx, knh_sfp_t *sfp);
-METHOD knh__Array_set3D(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_setAll(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opSubset(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opSubsete(Ctx *ctx, knh_sfp_t *sfp);
@@ -75,6 +69,13 @@ METHOD knh__Array_opItr(Ctx *ctx, knh_sfp_t *sfp);
 MAPPER knh_Iterator_Array(Ctx *ctx, knh_sfp_t *sfp);
 Mapper *knh_tMapper_newIteratorArray(Ctx *ctx, knh_class_t icid, knh_class_t acid);
 Mapper *knh_tMapper_newArrayIterator(Ctx *ctx, knh_class_t acid, knh_class_t icid);
+void knh_array_dfree(Ctx *ctx, void *ptr, size_t capacity, size_t usize);
+METHOD knh__Array_new__array2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_new__array3D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_get2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_get3D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_set2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Array_set3D(Ctx *ctx, knh_sfp_t *sfp);
 /* ../src/api/bytes.c */
 METHOD knh__Bytes_new(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Bytes_putc(Ctx *ctx, knh_sfp_t *sfp);
@@ -309,27 +310,17 @@ void knh_Context__dump(Ctx *ctx, Context *b, OutputStream *w, String *m);
 /* ../src/class/knh_AffineConv.c */
 AffineConv *new_AffineConv(Ctx *ctx, knh_float_t fa, knh_float_t fb);
 /* ../src/class/knh_Array.c */
+size_t knh_array_newsize(size_t newsize, size_t usize);
+size_t knh_array_index(Ctx *ctx, knh_int_t n, size_t size);
+void knh_Array_grow(Ctx *ctx, Array *a, size_t newsize, Object *v);
+void knh_IArray_grow(Ctx *ctx, IArray *a, size_t newsize, knh_integer_t v);
+void knh_FArray_grow(Ctx *ctx, FArray *a, size_t newsize, knh_float_t v);
 knh_Array_t* new_Array0(Ctx *ctx, size_t capacity);
 Object** knh_Array_list(Array *o);
 Object* knh_Array_last(Array *o);
 void knh_Array_subclear(Ctx *ctx, Array *o, size_t n);
-size_t knh_array_index(Ctx *ctx, knh_int_t n, size_t size);
 void knh_Array_remove(Ctx *ctx, Array *o, size_t n);
 Any* knh_Array_pop(Ctx *ctx, Array *o);
-/* ../src/class/knh_array_t.c */
-size_t knh_oarray_capacity(void *a);
-size_t knh_oarray_opsize(void *a);
-void knh_oarray_set_opsize(void *a, knh_short_t opsize);
-int knh_oarray_op1(void *a);
-void knh_oarray_setop1(void *a, knh_short_t op1);
-int knh_oarray_op2(void *a);
-void knh_oarray_setop2(void *a, knh_short_t op2);
-Object** knh_oarray_malloc(Ctx *ctx, size_t capacity, Object *value);
-Object **knh_oarray_resize(Ctx *ctx, Object **a, size_t newsize);
-Object** knh_oarray_grow(Ctx *ctx, Object **a);
-Object **knh_oarray_copy(Ctx *ctx, Object **a, size_t s, size_t offset);
-void knh_oarray_traverse(Ctx *ctx, Object **a, knh_ftraverse ftr);
-void knh_oarray_free(Ctx *ctx, knh_Object_t **a);
 /* ../src/class/knh_Bytes.c */
 size_t knh_bytes_newsize(size_t s);
 knh_bytes_t knh_BytesNULL_tobytes(Bytes *o);
@@ -344,12 +335,13 @@ int knh_bytes_parseinteger(knh_bytes_t t, knh_integer_t *value);
 int knh_bytes_parsefloat(knh_bytes_t t, knh_float_t *value);
 /* ../src/class/knh_BytesConv.c */
 size_t knh_BytesConv_conv(Ctx *ctx, BytesConv *o, knh_bytes_t t, knh_Bytes_t *ba);
-BytesConv* new_BytesConv(Ctx *ctx, String *name, f_bconv fbconv);
+BytesConv* new_BytesConv(Ctx *ctx, String *name, knh_fbcnv fbconv);
 BytesConv* new_BytesConv__iconv(Ctx *ctx, char *from, char *to);
 BytesConv* new_BytesConv__in(Ctx *ctx, char *from);
 BytesConv* new_BytesConv__out(Ctx *ctx, char *to);
-String *new_String__bconv(Ctx *ctx, knh_bytes_t t, BytesConv *bconv);
-String *new_String__wbufconv(Ctx *ctx, knh_wbuf_t cb, BytesConv *bconv);
+String *new_String__bconv(Ctx *ctx, knh_bytes_t t, BytesConv *bc);
+String *new_String__cwbconv(Ctx *ctx, knh_cwb_t cwb, BytesConv *bc);
+void knh_OutputStream_write__bconv(Ctx *ctx, OutputStream *w, knh_bytes_t t);
 void knh_BytesConv__k(Ctx *ctx, BytesConv *o, OutputStream *w, String *m);
 /* ../src/class/knh_Class.c */
 knh_class_t knh_tClass_newId(Ctx *ctx);
@@ -357,6 +349,7 @@ void konoha_setClassName(Ctx *ctx, knh_class_t cid, String *lname);
 char *knh_tClass_CLASSN(Ctx *ctx, knh_class_t cid);
 void konoha_setClassDefaultValue(Ctx *ctx, knh_class_t cid, Object *value, knh_fdefault fdefault);
 Object *konoha_getClassDefaultValue(Ctx *ctx, knh_class_t cid);
+Object *konoha_getDefaultValue(Ctx *ctx, knh_type_t type);
 void konoha_setClassParam(Ctx *ctx, knh_class_t cid, knh_class_t p1, knh_class_t p2);
 knh_class_t konoha_addGenericsClass(Ctx *ctx, knh_class_t cid, String *name, knh_class_t bcid, knh_class_t p1, knh_class_t p2);
 ClassStruct* new_ClassStruct0(Ctx *ctx, int field_size, int method_size);
@@ -378,30 +371,30 @@ knh_class_t knh_pmztype_toclass(Ctx *ctx, knh_type_t t, knh_class_t this_cid);
 knh_type_t knh_pmztype_totype(Ctx *ctx, knh_type_t t, knh_class_t this_cid);
 knh_class_t knh_class_Array(Ctx *ctx, knh_class_t p1);
 knh_class_t knh_class_Iterator(Ctx *ctx, knh_class_t p1);
-knh_class_t knh_pmzclass(Ctx *ctx, knh_class_t bcid, knh_class_t p1, knh_class_t p2);
+knh_class_t knh_class_Generics(Ctx *ctx, knh_class_t bcid, knh_class_t p1, knh_class_t p2);
 char* knh_format_type(Ctx *ctx, char *buf, size_t bufsiz, knh_type_t type);
 char *TYPEQ(knh_type_t type);
 char *knh_TYPEN(Ctx *ctx, knh_type_t type);
+/* ../src/class/knh_Closure.c */
+METHOD knh__Closure_new(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Closure_invoke(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__Closure_callcc(Ctx *ctx, knh_sfp_t *sfp);
+knh_class_t knh_class_Closure(Ctx *ctx, knh_type_t r0, knh_type_t p1, knh_type_t p2, knh_type_t p3);
+knh_class_t knh_class_MethodClosure(Ctx *ctx, knh_class_t cid, Method *mtd);
 /* ../src/class/knh_Connection.c */
 void knh_Connection_open(Ctx *ctx, Connection *o, String *urn);
 Connection* new_Connection(Ctx *ctx, String *urn);
 void knh_Connection_close(Ctx *ctx, Connection *o);
 /* ../src/class/knh_Context.c */
 void knh_Context_clearstack(Ctx *ctx);
-/* ../src/class/knh_Context_wbuf.c */
-knh_bytes_t knh_Bytes_firstbyte(Bytes *o, size_t pos);
-knh_bytes_t knh_Bytes_lastbyte(Bytes *o, size_t pos);
-size_t knh_Bytes_lastsize(Bytes *o, size_t pos);
-void knh_Bytes_subclear(Bytes *o, size_t pos);
-String *new_String__wbuf(Ctx *ctx, knh_wbuf_t cb);
-String *new_StringX__wbuf(Ctx *ctx, knh_class_t cid, knh_wbuf_t cb);
-knh_wbuf_t knh_Context_wbuf(Ctx* ctx);
-size_t knh_wbuf_size(knh_wbuf_t cb);
-knh_bytes_t knh_wbuf_tobytes(knh_wbuf_t cb);
-char *knh_wbuf_top(knh_wbuf_t cb);
-void knh_wbuf_clear(knh_wbuf_t cb);
-Bytes *knh_Context_openBConvBuf(Ctx* ctx);
-void knh_Context_closeBConvBuf(Ctx* ctx, Bytes *ba);
+/* ../src/class/knh_cwb_t.c */
+knh_cwb_t new_cwb(Ctx* ctx);
+size_t knh_cwb_size(knh_cwb_t cwb);
+knh_bytes_t knh_cwb_tobytes(knh_cwb_t cwb);
+char *knh_cwb_tochar(knh_cwb_t cwb);
+void knh_cwb_clear(knh_cwb_t cwb);
+String *new_String__cwb(Ctx *ctx, knh_cwb_t cwb);
+String *new_StringX__cwb(Ctx *ctx, knh_class_t cid, knh_cwb_t cwb);
 /* ../src/class/knh_DictIdx.c */
 DictIdx* new_DictIdx0(Ctx *ctx, size_t init, knh_int_t offset);
 DictIdx* new_DictIdx0__ignoreCase(Ctx *ctx, size_t init, knh_int_t offset);
@@ -578,10 +571,11 @@ void knh_write__ifmt(Ctx *ctx, OutputStream *w, char *fmt, knh_int_t n);
 void knh_write__ffmt(Ctx *ctx, OutputStream *w, char *fmt, knh_float_t n);
 void knh_write_integerfmt(Ctx *ctx, OutputStream *w, char *fmt, knh_integer_t n);
 void knh_write__flag(Ctx *ctx, OutputStream *w, knh_flag_t flag);
-void knh_write__type(Ctx *ctx, OutputStream *w, knh_type_t type);
-void knh_write__class(Ctx *ctx, OutputStream *w, knh_class_t cid);
-void knh_write__expt(Ctx *ctx, OutputStream *w, knh_expt_t cid);
-void knh_write__mn(Ctx *ctx, OutputStream *w, knh_methodn_t mn);
+void knh_write_cid(Ctx *ctx, OutputStream *w, knh_class_t cid);
+void knh_write_mn(Ctx *ctx, OutputStream *w, knh_methodn_t mn);
+void knh_write_cidmn(Ctx *ctx, OutputStream *w, knh_class_t cid, knh_methodn_t mn);
+void knh_write_type(Ctx *ctx, OutputStream *w, knh_type_t type);
+void knh_write_fline(Ctx *ctx, OutputStream *w, char *file, int line);
 void konoha_setverbose(int v);
 METHOD knh_fmethod_movableText(Ctx *ctx, knh_sfp_t *sfp);
 /* ../src/class/knh_Range.c */
@@ -731,6 +725,7 @@ knh_index_t knh_Asm_indexOfVariable(Asm *abr, knh_fieldn_t fnq);
 int TERMs_isTRUE(Stmt *stmt, size_t n);
 int TERMs_isFALSE(Stmt *stmt, size_t n);
 knh_type_t TERMs_gettype(Stmt *stmt, size_t n);
+char * knh_format_mtdparam(Ctx *ctx, char *buf, size_t bufsiz, Method *mtd, int n);
 void knh_Stmt_setType(Ctx *ctx, Stmt *stmt, knh_type_t type);
 Term * knh_StmtDECL_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns);
 Term *knh_StmtLET_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_type_t reqt);
@@ -803,7 +798,7 @@ Object *knh_tConst_systemValue(Ctx *ctx, int n);
 void konoha_addMapper(Ctx *ctx, knh_flag_t flag, knh_type_t stype, knh_type_t ttype, knh_fmapper fmap, Object *mapdata);
 MAPPER knh_Mapper_fInterface(Ctx *ctx, knh_sfp_t *sfp);
 MAPPER knh_fmapper_null(Ctx *ctx, knh_sfp_t *sfp);
-knh_bool_t knh_Mapper_isNoSuchMapping(Mapper *o);
+knh_bool_t knh_Mapper_isNoSuchMapping(Mapper *mpr);
 Mapper *knh_tMapper_find_(Ctx *ctx, knh_class_t scid, knh_class_t tcid, int isgen);
 /* ../src/main/knh_tMethod.c */
 void knh_Class_addMethod(Ctx *ctx, knh_class_t cid, Method *mtd);
@@ -811,21 +806,10 @@ void knh_tMethodField_add(Ctx *ctx, MethodField *mf);
 void knh_tMethodField_add0(Ctx *ctx, knh_flag_t flag, knh_type_t rtype);
 void knh_tMethodField_add1(Ctx *ctx, knh_flag_t flag, knh_type_t rtype, knh_type_t t1, knh_fieldn_t fn1);
 MethodField *knh_tMethodField(Ctx *ctx, size_t n);
+MethodField *knh_tMethodField_find0(Ctx *ctx, knh_type_t rtype);
 Method* knh_Class_getMethod__(Ctx *ctx, knh_class_t this_cid, knh_methodn_t mn, knh_bool_t gen);
 Method *knh_Method_ufind(Ctx *ctx, knh_class_t cid, knh_methodn_t mn);
 Method *knh_tMethod_findMT(Ctx *ctx, knh_class_t cid, knh_methodn_t mn);
-/* ../src/main/knh_tName.c */
-int knh_bytes_isOptionalMT(knh_bytes_t t);
-knh_bytes_t knh_bytes_mtoption(knh_bytes_t t);
-char *knh_format_newFMT(char *buf, size_t bufsiz, knh_bytes_t t, int dot, char *fmt);
-String *knh_tfieldn_name(Ctx *ctx, knh_fieldn_t fn);
-knh_fieldn_t knh_tName_get_fnq(Ctx *ctx, knh_bytes_t tname, knh_fieldn_t def);
-String *new_String__mn(Ctx *ctx, knh_methodn_t mn);
-knh_methodn_t knh_tName_getMethodn(Ctx *ctx, knh_bytes_t tname, knh_methodn_t def);
-char * knh_format_methodn(Ctx *ctx, char *buf, size_t bufsiz, knh_methodn_t mn);
-char * knh_format_cmethodn(Ctx *ctx, char *buf, size_t bufsiz, knh_class_t cid, knh_methodn_t mn);
-knh_fileid_t knh_tName_getFileId(Ctx *ctx, knh_bytes_t t);
-String *knh_tfileid_name(Ctx *ctx, knh_fileid_t fileid);
 /* ../src/main/knh_tPackage.c */
 void KNH_TDRVAPI(Ctx *ctx, char *alias, knh_drvapi_t* p);
 knh_drvapi_t *knh_System_getDRVAPI(Ctx *ctx, int type, knh_bytes_t name);
@@ -870,9 +854,16 @@ void knh_fastfree(Ctx *ctx, void *block, size_t size);
 knh_Object_t *new_hObject(Ctx *ctx, knh_flag_t flag, knh_class_t bcid, knh_class_t cid);
 knh_Object_t *new_Object_bcid(Ctx *ctx, knh_class_t bcid, int init);
 knh_Object_t *new_Object_init(Ctx *ctx, knh_flag_t flag, knh_class_t cid, int init);
-void knh_Object_RCsweep(Ctx *ctx, Object *o);
 void knh_Object_free(Ctx *ctx, knh_Object_t *o);
 void knh_Object_traverse(Ctx *ctx, knh_Object_t *o, knh_ftraverse ftr);
+void knh_fgchook_nop(Ctx *ctx);
+void knh_fgchook_collect(Ctx *ctx);
+void knh_Object_mark0(Ctx *ctx, Object *o);
+void knh_Object_sweep0(Ctx *ctx, Object *o);
+void knh_Object_mark1(Ctx *ctx, Object *o);
+void knh_Object_sweep1(Ctx *ctx, Object *o);
+void knh_Object_RCsweep(Ctx *ctx, Object *o);
+knh_ftraverse konoha_getDefaultSweepFunction();
 /* ../src/main/konoha_pkgapi.c */
 /* ../src/main/konoha_print.c */
 void knh_stack_pmsg(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, String *s);
@@ -880,9 +871,20 @@ void knh_stack_p(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, knh_methodn_t mn, in
 int knh_verbose(void);
 /* ../src/main/konoha_stack.c */
 int knh_sfp_argc(Ctx *ctx, knh_sfp_t *varg);
-void knh_dynamiccall(Ctx *ctx, knh_sfp_t *mbp);
+char * knh_format_mtdparam(Ctx *ctx, char *buf, size_t bufsiz, Method *mtd, int n); void knh_sfp_typecheck(Ctx *ctx, knh_sfp_t *sfp, knh_type_t reqt, Method *mtd, knh_code_t *pc);
 void knh_sfp_dump(Ctx *ctx, knh_sfp_t *sfp);
-void knh_Context__s(Ctx *ctx, Context *o, OutputStream *w, String *m);
+/* ../src/main/konoha_system.c */
+int knh_bytes_isOptionalMT(knh_bytes_t t);
+knh_bytes_t knh_bytes_mtoption(knh_bytes_t t);
+char *knh_format_newFMT(char *buf, size_t bufsiz, knh_bytes_t t, int dot, char *fmt);
+String *knh_tfieldn_name(Ctx *ctx, knh_fieldn_t fn);
+knh_fieldn_t knh_tName_get_fnq(Ctx *ctx, knh_bytes_t tname, knh_fieldn_t def);
+String *new_String__mn(Ctx *ctx, knh_methodn_t mn);
+knh_methodn_t knh_tName_getMethodn(Ctx *ctx, knh_bytes_t tname, knh_methodn_t def);
+char *knh_format_methodn(Ctx *ctx, char *buf, size_t bufsiz, knh_methodn_t mn);
+char * knh_format_cmethodn(Ctx *ctx, char *buf, size_t bufsiz, knh_class_t cid, knh_methodn_t mn);
+knh_fileid_t knh_tName_getFileId(Ctx *ctx, knh_bytes_t t);
+String *knh_tfileid_name(Ctx *ctx, knh_fileid_t fileid);
 /* ../src/main/konoha_table.c */
 void konoha_initContext(Ctx *ctx, knh_Context_t *o, size_t stacksize);
 void knh_Context_traverse(Ctx *ctx, knh_Context_t *o, knh_ftraverse ftr);
