@@ -50,8 +50,8 @@ knh_code_t* KNH_ASM_XMOVDEF_(Ctx *ctx, Asm *o,knh_sfx_t a1,knh_class_t a2);
 knh_code_t* KNH_ASM_XMOVSYS_(Ctx *ctx, Asm *o,knh_sfx_t a1,knh_ushort_t a2);
 knh_code_t* KNH_ASM_SWAP_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2);
 knh_code_t* KNH_ASM_PINIo_(Ctx *ctx, Asm *o,knh_sfi_t a1,Object* a2);
-knh_code_t* KNH_ASM_SETEBP_(Ctx *ctx, Asm *o,knh_sfi_t a1);
-knh_code_t* KNH_ASM_CHECKEBP_(Ctx *ctx, Asm *o,knh_sfi_t a1);
+knh_code_t* KNH_ASM_SETESP_(Ctx *ctx, Asm *o,knh_sfi_t a1);
+knh_code_t* KNH_ASM_CHECKESP_(Ctx *ctx, Asm *o,knh_sfi_t a1);
 knh_code_t* KNH_ASM_RET_(Ctx *ctx, Asm *o);
 knh_code_t* KNH_ASM_RETn_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2);
 knh_code_t* KNH_ASM_RETa_(Ctx *ctx, Asm *o,knh_sfi_t a1,knh_sfi_t a2);
@@ -333,7 +333,7 @@ typedef struct {
 
 #define OPSIZE_PINIO  (sizeof(klr_pinio_t)-sizeof(void*))
 
-#define OPCODE_SETEBP ((knh_opcode_t)14)
+#define OPCODE_SETESP ((knh_opcode_t)14)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -341,11 +341,11 @@ typedef struct {
 	knh_opcode_t opcode;
 	knh_sfi_t a1;
 	void *nextaddr;
-} klr_setebp_t;
+} klr_setesp_t;
 
-#define OPSIZE_SETEBP  (sizeof(klr_setebp_t)-sizeof(void*))
+#define OPSIZE_SETESP  (sizeof(klr_setesp_t)-sizeof(void*))
 
-#define OPCODE_CHECKEBP ((knh_opcode_t)15)
+#define OPCODE_CHECKESP ((knh_opcode_t)15)
 typedef struct {
 #ifdef KNH_USING_THREADEDCODE
 	void *codeaddr;
@@ -353,9 +353,9 @@ typedef struct {
 	knh_opcode_t opcode;
 	knh_sfi_t a1;
 	void *nextaddr;
-} klr_checkebp_t;
+} klr_checkesp_t;
 
-#define OPSIZE_CHECKEBP  (sizeof(klr_checkebp_t)-sizeof(void*))
+#define OPSIZE_CHECKESP  (sizeof(klr_checkesp_t)-sizeof(void*))
 
 #define OPCODE_RET ((knh_opcode_t)16)
 typedef struct {
