@@ -79,6 +79,8 @@ static char* StructName[] =  {
 	"Tuple2",
 	"Range",
 	"Array",
+	"IArray",
+	"FArray",
 	"Iterator",
 	"DictMap",
 	"DictSet",
@@ -224,6 +226,24 @@ static knh_StructData_t StructData[] = {
 		(knh_fstruct_hashCode)knh_Array_hashCode,
 		(knh_fstruct_newClass)knh_Array_newClass
 	},
+	{
+		"IArray", STRUCT_IArray, FLAG_IArray, 0,
+		(knh_fstruct_init)knh_IArray_init,
+		(knh_fstruct_copy)knh_IArray_copy, 
+		(knh_fstruct_traverse)knh_IArray_traverse,
+		(knh_fstruct_compareTo)knh_IArray_compareTo,
+		(knh_fstruct_hashCode)knh_IArray_hashCode,
+		(knh_fstruct_newClass)knh_IArray_newClass
+	},
+	{
+		"FArray", STRUCT_FArray, FLAG_FArray, 0,
+		(knh_fstruct_init)knh_FArray_init,
+		(knh_fstruct_copy)knh_FArray_copy, 
+		(knh_fstruct_traverse)knh_FArray_traverse,
+		(knh_fstruct_compareTo)knh_FArray_compareTo,
+		(knh_fstruct_hashCode)knh_FArray_hashCode,
+		(knh_fstruct_newClass)knh_FArray_newClass
+	},
 	{
 		"Iterator", STRUCT_Iterator, FLAG_Iterator, sizeof(knh_Iterator_struct),
 		(knh_fstruct_init)knh_Iterator_init,
@@ -554,6 +574,8 @@ static knh_ClassData_t ClassData[] = {
 	{"konoha.Tuple2", CLASS_Tuple2, CFLAG_Tuple2, STRUCT_Tuple2, CLASS_Object, 0, 0, 0, knh_Tuple2_fdefault},
 	{"konoha.Range", CLASS_Range, CFLAG_Range, STRUCT_Range, CLASS_Object, 0, 0, 0, knh_Range_fdefault},
 	{"konoha.Array", CLASS_Array, CFLAG_Array, STRUCT_Array, CLASS_Object, 0, 31, 0, knh_Array_fdefault},
+	{"konoha.IArray", CLASS_IArray, CFLAG_IArray, STRUCT_IArray, CLASS_Object, 0, 0, 0, knh_IArray_fdefault},
+	{"konoha.FArray", CLASS_FArray, CFLAG_FArray, STRUCT_FArray, CLASS_Object, 0, 0, 0, knh_FArray_fdefault},
 	{"konoha.Iterator", CLASS_Iterator, CFLAG_Iterator, STRUCT_Iterator, CLASS_Object, 0, 2, 0, knh_Iterator_fdefault},
 	{"konoha.DictMap", CLASS_DictMap, CFLAG_DictMap, STRUCT_DictMap, CLASS_Object, 0, 12, 0, knh_DictMap_fdefault},
 	{"konoha.DictSet", CLASS_DictSet, CFLAG_DictSet, STRUCT_DictSet, CLASS_Object, 0, 13, 0, knh_DictSet_fdefault},
@@ -567,7 +589,7 @@ static knh_ClassData_t ClassData[] = {
 	{"konoha.Method", CLASS_Method, CFLAG_Method, STRUCT_Method, CLASS_Object, 0, 15, 0, knh_Method_fdefault},
 	{"konoha.Mapper", CLASS_Mapper, CFLAG_Mapper, STRUCT_Mapper, CLASS_Object, 0, 12, 0, knh_Mapper_fdefault},
 	{"konoha.ClassMap", CLASS_ClassMap, CFLAG_ClassMap, STRUCT_ClassMap, CLASS_Object, 0, 1, 0, knh_ClassMap_fdefault},
-	{"konoha.Closure", CLASS_Closure, CFLAG_Closure, STRUCT_Closure, CLASS_Object, 0, 0, 0, knh_Closure_fdefault},
+	{"konoha.Closure", CLASS_Closure, CFLAG_Closure, STRUCT_Closure, CLASS_Object, 0, 3, 0, knh_Closure_fdefault},
 	{"konoha.AffineConv", CLASS_AffineConv, CFLAG_AffineConv, STRUCT_AffineConv, CLASS_Object, 0, 0, 0, knh_AffineConv_fdefault},
 	{"konoha.IntUnit", CLASS_IntUnit, CFLAG_IntUnit, STRUCT_IntUnit, CLASS_Object, 0, 1, 0, knh_IntUnit_fdefault},
 	{"konoha.FloatUnit", CLASS_FloatUnit, CFLAG_FloatUnit, STRUCT_FloatUnit, CLASS_Object, 0, 0, 0, knh_FloatUnit_fdefault},
@@ -583,7 +605,7 @@ static knh_ClassData_t ClassData[] = {
 	{"konoha.Script", CLASS_Script, CFLAG_Script, STRUCT_Script, CLASS_Object, 0, 6, 0, knh_Script_fdefault},
 	{"konoha.NameSpace", CLASS_NameSpace, CFLAG_NameSpace, STRUCT_NameSpace, CLASS_Object, 0, 0, 0, knh_NameSpace_fdefault},
 	{"konoha.System", CLASS_System, CFLAG_System, STRUCT_System, CLASS_Object, 0, 15, 0, knh_System_fdefault},
-	{"konoha.Context", CLASS_Context, CFLAG_Context, STRUCT_Context, CLASS_Object, 0, 17, 0, knh_Context_fdefault},
+	{"konoha.Context", CLASS_Context, CFLAG_Context, STRUCT_Context, CLASS_Object, 0, 16, 0, knh_Context_fdefault},
 	{"konoha.Token", CLASS_Token, CFLAG_Token, STRUCT_Token, CLASS_Object, 0, 3, 0, knh_Token_fdefault},
 	{"konoha.Stmt", CLASS_Stmt, CFLAG_Stmt, STRUCT_Stmt, CLASS_Object, 0, 2, 0, knh_Stmt_fdefault},
 	{"konoha.Asm", CLASS_Asm, CFLAG_Asm, STRUCT_Asm, CLASS_Object, 0, 0, 0, knh_Asm_fdefault},
@@ -647,9 +669,11 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"autoFlush", FIELDN_autoFlush},
 	{"b", FIELDN_b},
 	{"bag", FIELDN_bag},
+	{"base", FIELDN_base},
 	{"bits", FIELDN_bits},
 	{"buf", FIELDN_buf},
 	{"c", FIELDN_c},
+	{"callcc", FIELDN_callcc},
 	{"class", FIELDN_class},
 	{"clear", FIELDN_clear},
 	{"close", FIELDN_close},
@@ -660,6 +684,7 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"debug", FIELDN_debug},
 	{"default", FIELDN_default},
 	{"defined", FIELDN_defined},
+	{"delegate", FIELDN_delegate},
 	{"delim", FIELDN_delim},
 	{"dump", FIELDN_dump},
 	{"e", FIELDN_e},
@@ -694,6 +719,7 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"init", FIELDN_init},
 	{"insert", FIELDN_insert},
 	{"int", FIELDN_int},
+	{"invoke", FIELDN_invoke},
 	{"isAbstract", FIELDN_isAbstract},
 	{"isAspect", FIELDN_isAspect},
 	{"isAutoFlush", FIELDN_isAutoFlush},
@@ -750,6 +776,7 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"mode", FIELDN_mode},
 	{"modified", FIELDN_modified},
 	{"msg", FIELDN_msg},
+	{"mtd", FIELDN_mtd},
 	{"n", FIELDN_n},
 	{"name", FIELDN_name},
 	{"new", FIELDN_new},
@@ -800,6 +827,7 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"pop", FIELDN_pop},
 	{"print", FIELDN_print},
 	{"println", FIELDN_println},
+	{"proceed", FIELDN_proceed},
 	{"prompt", FIELDN_prompt},
 	{"property", FIELDN_property},
 	{"putc", FIELDN_putc},
@@ -824,6 +852,7 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"stdErr", FIELDN_stdErr},
 	{"storingBuffer", FIELDN_storingBuffer},
 	{"string", FIELDN_string},
+	{"super", FIELDN_super},
 	{"swap", FIELDN_swap},
 	{"t", FIELDN_t},
 	{"term", FIELDN_term},
@@ -855,18 +884,12 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_void_OutputStream_Any, 2+ 0, TYPE_void, TYPE_OutputStream, FIELDN_w, TYPE_Any, FIELDN_o},
 	{MFN_Any, 0+ 0, TYPE_Any},
 	{MFN_nThis_Int, 1+ 0, NNTYPE_This, TYPE_Int, FIELDN_i},
-	{MFN_nThis_nInt_nInt, 2+ 0, NNTYPE_This, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y},
-	{MFN_nThis_nInt_nInt_nInt, 3+ 0, NNTYPE_This, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z},
 	{MFN_nThis_Any1_, 1+ 0, NNTYPE_This, TYPE_Any1, FIELDN_v},
 	{MFN_void_Any1, 1+ 0, TYPE_void, TYPE_Any1, FIELDN_v},
 	{MFN_void_Any1_, 1+ 0, TYPE_void, TYPE_Any1, FIELDN_v},
 	{MFN_nInt, 0+ 0, NNTYPE_Int},
 	{MFN_Any1_nInt, 1+ 0, TYPE_Any1, NNTYPE_Int, FIELDN_n},
-	{MFN_Any1_nInt_nInt, 2+ 0, TYPE_Any1, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y},
-	{MFN_Any1_nInt_nInt_nInt, 3+ 0, TYPE_Any1, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z},
 	{MFN_void_nInt_Any1, 2+ 0, TYPE_void, NNTYPE_Int, FIELDN_n, TYPE_Any1, FIELDN_v},
-	{MFN_void_nInt_nInt_Any1, 3+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, TYPE_Any1, FIELDN_v},
-	{MFN_void_nInt_nInt_nInt_Any1, 4+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z, TYPE_Any1, FIELDN_v},
 	{MFN_nThis_Int_Int, 2+ 0, NNTYPE_This, TYPE_Int, FIELDN_s, TYPE_Int, FIELDN_e},
 	{MFN_void_nInt, 1+ 0, TYPE_void, NNTYPE_Int, FIELDN_n},
 	{MFN_Any1, 0+ 0, TYPE_Any1},
@@ -875,6 +898,12 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_nBoolean_Any1, 1+ 0, NNTYPE_Boolean, TYPE_Any1, FIELDN_v},
 	{MFN_void_nInt_nInt, 2+ 0, TYPE_void, NNTYPE_Int, FIELDN_m, NNTYPE_Int, FIELDN_n},
 	{MFN_iAny1, 0+ 0, NNITYPE_Any1},
+	{MFN_nThis_nInt_nInt, 2+ 0, NNTYPE_This, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y},
+	{MFN_nThis_nInt_nInt_nInt, 3+ 0, NNTYPE_This, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z},
+	{MFN_Any1_nInt_nInt, 2+ 0, TYPE_Any1, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y},
+	{MFN_Any1_nInt_nInt_nInt, 3+ 0, TYPE_Any1, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z},
+	{MFN_void_nInt_nInt_Any1, 3+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, TYPE_Any1, FIELDN_v},
+	{MFN_void_nInt_nInt_nInt_Any1, 4+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z, TYPE_Any1, FIELDN_v},
 	{MFN_nBytes_Int, 1+ 0, NNTYPE_Bytes, TYPE_Int, FIELDN_i},
 	{MFN_void_nBytes_Int_Int, 3+ 0, TYPE_void, NNTYPE_Bytes, FIELDN_b, TYPE_Int, FIELDN_n, TYPE_Int, FIELDN_l},
 	{MFN_void_nInt_, 1+ 0, TYPE_void, NNTYPE_Int, FIELDN_v},
@@ -934,6 +963,8 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_Any_nString, 1+ 0, TYPE_Any, NNTYPE_String, FIELDN_k},
 	{MFN_void_nString_Any, 2+ 0, TYPE_void, NNTYPE_String, FIELDN_k, TYPE_Any, FIELDN_v},
 	{MFN_naString_String, 1+ 0, NNATYPE_String, TYPE_String, FIELDN_k},
+	{MFN_nThis_nAny_nMethod, 2+ 0, NNTYPE_This, NNTYPE_Any, FIELDN_b, NNTYPE_Method, FIELDN_m},
+	{MFN_Any0_Any1_Any2_Any3, 3+ 0, TYPE_Any0, TYPE_Any1, FIELDN_x, TYPE_Any2, FIELDN_y, TYPE_Any3, FIELDN_z},
 	{MFN_nException_String_String_Object, 3+ 0, NNTYPE_Exception, TYPE_String, FIELDN_e, TYPE_String, FIELDN_m, TYPE_Object, FIELDN_b},
 	{MFN_String, 0+ 0, TYPE_String},
 	{MFN_nThis, 0+ 0, NNTYPE_This},
@@ -949,18 +980,12 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 static knh_MethodData_t MethodData[] = {
 	{knh__Array_new, 0, CLASS_Array, METHODN_new, 0, MFN_nThis_Int, (void*)NULL},
 	{knh__Array_new__array, 0, CLASS_Array, METHODN_new__array, 0, MFN_nThis_Int, (void*)NULL},
-	{knh__Array_new__array2D, 0, CLASS_Array, METHODN_new__array2D, 0, MFN_nThis_nInt_nInt, (void*)NULL},
-	{knh__Array_new__array3D, 0, CLASS_Array, METHODN_new__array3D, 0, MFN_nThis_nInt_nInt_nInt, (void*)NULL},
 	{knh__Array_new__init, KNH_FLAG_MF_VARARGS, CLASS_Array, METHODN_new__init, 0, MFN_nThis_Any1_, (void*)NULL},
 	{knh__Array_add, 0, CLASS_Array, METHODN_add, 0, MFN_void_Any1, (void*)NULL},
 	{knh__Array_opLshift, KNH_FLAG_MF_VARARGS, CLASS_Array, METHODN_opLshift, 0, MFN_void_Any1_, (void*)NULL},
 	{knh__Array_opSize, 0, CLASS_Array, METHODN_opSize, 0, MFN_nInt, (void*)NULL},
 	{knh__Array_get, 0, CLASS_Array, METHODN_get, 0, MFN_Any1_nInt, (void*)NULL},
-	{knh__Array_get2D, 0, CLASS_Array, METHODN_get2D, 0, MFN_Any1_nInt_nInt, (void*)NULL},
-	{knh__Array_get3D, 0, CLASS_Array, METHODN_get3D, 0, MFN_Any1_nInt_nInt_nInt, (void*)NULL},
 	{knh__Array_set, 0, CLASS_Array, METHODN_set, 0, MFN_void_nInt_Any1, (void*)NULL},
-	{knh__Array_set2D, 0, CLASS_Array, METHODN_set2D, 0, MFN_void_nInt_nInt_Any1, (void*)NULL},
-	{knh__Array_set3D, 0, CLASS_Array, METHODN_set3D, 0, MFN_void_nInt_nInt_nInt_Any1, (void*)NULL},
 	{knh__Array_setAll, 0, CLASS_Array, METHODN_setAll, 0, MFN_void_Any1, (void*)NULL},
 	{knh__Array_opSubset, 0, CLASS_Array, METHODN_opSubset, 0, MFN_nThis_Int_Int, (void*)NULL},
 	{knh__Array_opSubsete, 0, CLASS_Array, METHODN_opSubsete, 0, MFN_nThis_Int_Int, (void*)NULL},
@@ -978,6 +1003,12 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Array_shuffle, 0, CLASS_Array, METHODN_shuffle, 0, MFN_void, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_Array, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Array__k},
 	{knh__Array_opItr, 0, CLASS_Array, METHODN_opItr, 0, MFN_iAny1, (void*)NULL},
+	{knh__Array_new__array2D, 0, CLASS_Array, METHODN_new__array2D, 0, MFN_nThis_nInt_nInt, (void*)NULL},
+	{knh__Array_new__array3D, 0, CLASS_Array, METHODN_new__array3D, 0, MFN_nThis_nInt_nInt_nInt, (void*)NULL},
+	{knh__Array_get2D, 0, CLASS_Array, METHODN_get2D, 0, MFN_Any1_nInt_nInt, (void*)NULL},
+	{knh__Array_get3D, 0, CLASS_Array, METHODN_get3D, 0, MFN_Any1_nInt_nInt_nInt, (void*)NULL},
+	{knh__Array_set2D, 0, CLASS_Array, METHODN_set2D, 0, MFN_void_nInt_nInt_Any1, (void*)NULL},
+	{knh__Array_set3D, 0, CLASS_Array, METHODN_set3D, 0, MFN_void_nInt_nInt_nInt_Any1, (void*)NULL},
 	{knh__Bytes_new, 0, CLASS_Bytes, METHODN_new, 0, MFN_nBytes_Int, (void*)NULL},
 	{knh__Bytes_putc, 0, CLASS_Bytes, METHODN_putc, 0, MFN_void_nInt, (void*)NULL},
 	{knh__Bytes_write, 0, CLASS_Bytes, METHODN_write, 0, MFN_void_nBytes_Int_Int, (void*)NULL},
@@ -1173,6 +1204,9 @@ static knh_MethodData_t MethodData[] = {
 	{knh_fmethod_movableText, 0, CLASS_Context, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Context__dump},
 	{knh_fmethod_movableText, 0, CLASS_BytesConv, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_BytesConv__k},
 	{knh_fmethod_movableText, 0, CLASS_ClassMap, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_ClassMap__dump},
+	{knh__Closure_new, 0, CLASS_Closure, METHODN_new, 0, MFN_nThis_nAny_nMethod, (void*)NULL},
+	{knh__Closure_invoke, 0, CLASS_Closure, METHODN_invoke, 0, MFN_Any0_Any1_Any2_Any3, (void*)NULL},
+	{knh__Closure_callcc, 0, CLASS_Closure, METHODN_callcc, 0, MFN_Any0_Any1_Any2_Any3, (void*)NULL},
 	{knh__Connection_close, 0, CLASS_Connection, METHODN_close, 0, MFN_void, (void*)NULL},
 	{knh__DictIdx_size, 0, CLASS_DictIdx, METHODN_size, 0, MFN_nInt, (void*)NULL},
 	{knh__DictIdx_index, 0, CLASS_DictIdx, METHODN_index, 0, MFN_nInt_nString, (void*)NULL},
@@ -1210,7 +1244,6 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Script_isStatement, 0, CLASS_Script, METHODN_isStatement, 0, MFN_nBoolean_nString, (void*)NULL},
 	{knh__Script_readLine, 0, CLASS_Script, METHODN_readLine, 0, MFN_String_String, (void*)NULL},
 	{knh__Script_addHistory, 0, CLASS_Script, METHODN_addHistory, 0, MFN_void_nString, (void*)NULL},
-	{knh_fmethod_movableText, 0, CLASS_Context, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Context__s},
 	{knh__Object_opInstanceof, 0, CLASS_Object, METHODN_opInstanceof, 0, MFN_Boolean_nClass, (void*)NULL},
 	{knh__Object_isRelease, 0, CLASS_Object, METHODN_isRelease, 0, MFN_nBoolean, (void*)NULL},
 	{knh__Object_isDebug, 0, CLASS_Object, METHODN_isDebug, 0, MFN_nBoolean, (void*)NULL},
