@@ -62,8 +62,6 @@ extern "C" {
 #define KNH_MT_VOLATILE
 #endif
 
-/* ------------------------------------------------------------------------ */
-
 /* ------------------------------------------------------------------------ */
 /* Bool(ean), knh_bool_t */
 /* ------------------------------------------------------------------------ */
@@ -198,6 +196,9 @@ typedef knh_ushort_t              knh_flag_t;    /* flag field */
 	#endif
 #endif
 
+#define KNH_FLOAT_NAN             (0.0/0.0)
+#define KNH_FLOAT_NULL            KNH_FLOAT_NAN
+
 #define knh_double_t                    knh_float_t /* just in case */
 #define KNH_FLOAT_FMTSIZ                256
 
@@ -228,11 +229,12 @@ typedef unsigned long long int         knh_uint64_t;
 
 #ifdef KNH_USING_INT32
 
-#define knh_integer_t    knh_int_t
-#define knh_uinteger_t   knh_uint_t
+typedef knh_int_t        knh_integer_t;
+typedef knh_uint_t       knh_uinteger_t;
 
 #define KNH_INTEGER_MAX               LONG_MAX
-#define KNH_INTEGER_MIN               LONG_MIN
+#define KNH_INTEGER_MIN               (LONG_MIN + 1)
+#define KNH_INTEGER_NULL              (LONG_MIN)
 #define KNH_INTEGER_FMT               "%ld"
 #define KNH_INTEGER_XFMT              "%lx"
 #define KNH_INTEGER_BSIZE             (sizeof(knh_integer_t))
@@ -244,11 +246,12 @@ typedef unsigned long long int         knh_uint64_t;
 
 #else /*KNH_USING_INT32*/
 
-#define knh_integer_t    knh_int64_t
-#define knh_uinteger_t   knh_uint64_t
+typedef knh_int64_t        knh_integer_t;
+typedef knh_uint64_t       knh_uinteger_t;
 
 #define KNH_INTEGER_MAX               LLONG_MAX
-#define KNH_INTEGER_MIN               LLONG_MIN
+#define KNH_INTEGER_MIN               (LLONG_MIN + 1)
+#define KNH_INTEGER_NULL              (LLONG_MIN)
 #define KNH_INTEGER_FMT               "%lld"
 #define KNH_INTEGER_XFMT              "%llx"
 #define KNH_INTEGER_BSIZE             (sizeof(knh_integer_t))
@@ -294,6 +297,45 @@ typedef knh_ushort_t          knh_fileid_t;
 typedef knh_ushort_t          knh_fieldn_t;
 typedef knh_ushort_t          knh_methodn_t;
 
+#define FIELDN_NONAME    ((knh_fieldn_t)-1)
+#define FIELDN_NEWID     ((knh_fieldn_t)-2)
+#define FIELDN_return    FIELDN_
+#define METHODN_NONAME   ((knh_methodn_t)-1)
+#define METHODN_NEWID    ((knh_methodn_t)-2)
+
+#define KNH_FLAG_FN_U1         KNH_FLAG_T0
+#define KNH_FLAG_FN_U2         KNH_FLAG_T1
+#define KNH_FLAG_FN_SUPER      (KNH_FLAG_T0|KNH_FLAG_T1)
+
+#define FIELDN_IS_SUPER(fnq)       ((fnq & KNH_FLAG_FN_SUPER) == KNH_FLAG_FN_SUPER)
+#define FIELDN_IS_U1(fnq)          ((fnq & KNH_FLAG_FN_U1) == KNH_FLAG_FN_U1)
+#define FIELDN_IS_U2(fnq)          ((fnq & KNH_FLAG_FN_U2) == KNH_FLAG_FN_U2)
+
+#define FIELDN_UNMASK(fnq)         (fnq & (~(KNH_FLAG_FN_SUPER|KNH_FLAG_FN_U1|KNH_FLAG_FN_U2)))
+
+//#define _METHODN_NONAME   KONOHA_TCLASS_SIZE
+//#define _METHODN_NEWID    0
+
+// 0000 0000 0000 0000
+// 0100 0000 0000 0000
+// 0010 0000 0000 0000
+// 0110 0000 0000 0000
+
+#define KNH_FLAG_MN_SUPER        KNH_FLAG_T0
+#define KNH_FLAG_MN_GETTER       KNH_FLAG_T1
+#define KNH_FLAG_MN_SETTER       KNH_FLAG_T2
+#define KNH_FLAG_MN_MOVTEXT      (KNH_FLAG_T1|KNH_FLAG_T2)
+#define KNH_FLAG_MN_FIELDN       (~(KNH_FLAG_T0|KNH_FLAG_T1|KNH_FLAG_T2))
+
+#define METHODN_IS_GETTER(mn)   ((mn & KNH_FLAG_MN_MOVTEXT) == KNH_FLAG_MN_GETTER)
+#define METHODN_TO_GETTER(mn)   (mn | KNH_FLAG_MN_GETTER)
+#define METHODN_IS_SETTER(mn)   ((mn & KNH_FLAG_MN_MOVTEXT) == KNH_FLAG_MN_SETTER)
+#define METHODN_TO_SETTER(mn)   (mn | KNH_FLAG_MN_SETTER)
+#define METHODN_IS_MOVTEXT(mn)  ((mn & KNH_FLAG_MN_MOVTEXT) == KNH_FLAG_MN_MOVTEXT)
+#define METHODN_TO_MOVTEXT(mn)  (mn | KNH_FLAG_MN_MOVTEXT)
+
+#define METHODN_TOFIELDN(mn)     (mn & KNH_FLAG_MN_FIELDN)
+
 /* ------------------------------------------------------------------------ */
 /* Object */
 /* ------------------------------------------------------------------------ */
@@ -315,20 +357,24 @@ typedef struct knh_hObject_t {
 #endif
 } knh_hObject_t ;
 
+typedef struct knh_fObject_t {
+	void *meta;
+	struct knh_Object_t *recentCreatedObject;
+} knh_fObject_t ;
+
+#define SIZEOF_FOBJECT_T  sizeof(knh_fObject_t)
+
 typedef struct knh_Object_t {
 	knh_hObject_t h;
 	void *ref;
 	void *ref2_unused;
 	void *ref3_unused;
+	knh_fObject_t f;
 } knh_Object_t ;
 
 
 #ifdef KNH_OBJECT_MAGIC
-	#ifdef KNH_FASTMODE
-		#define KNH_ASSERT_ISOBJECT(o)
-	#else
-		#define KNH_ASSERT_ISOBJECT(o)        KNH_ASSERT((o)->h.magic == KNH_OBJECT_MAGIC)
-	#endif
+	#define KNH_ASSERT_ISOBJECT(o)        DBG2_ASSERT((o)->h.magic == KNH_OBJECT_MAGIC)
 #else/*KONOHA_OBJECT_MAGIC*/
 	#define KNH_ASSERT_ISOBJECT(o)
 #endif/*KONOHA_OBJECT_MAGIC*/
@@ -345,7 +391,6 @@ typedef struct knh_Object_t {
 
 #define KNH_FIELDn(v,n)            ((Script*)(v))->fields[(n)]
 
-
 /* ------------------------------------------------------------------------ */
 /* Context */
 /* ------------------------------------------------------------------------ */
@@ -379,6 +424,7 @@ typedef struct knh_sfp_t {
 		struct knh_String_t *s;
 		struct knh_Bytes_t  *ba;
 		struct knh_Iterator_t *it;
+		struct knh_Closure_t *cc;
 		struct knh_Glue_t   *n;
 		struct knh_OutputStream_t *w;
 		struct knh_Method_t *mtd;
@@ -390,9 +436,8 @@ typedef struct knh_sfp_t {
 		knh_boolean_t bvalue;
 		knh_integer_t ivalue;
 		knh_float_t   fvalue;
-		knh_code_t    *pc;
-		knh_uint_t     op;
 		knh_uint64_t   data;
+		knh_code_t       *pc;
 	};
 } knh_sfp_t;
 
@@ -486,6 +531,8 @@ typedef const struct knh_Context_t    Ctx;
 #define KNH_FASTMALLOC_SIZE  (sizeof(knh_Object_t) - sizeof(void*))
 #define KNH_FASTMALLOC_BSIZE (KNH_FASTMALLOC_SIZE/sizeof(knh_Object_t*))
 
+#define KNH_SMALLMALLOC_SIZE 80
+
 /* ------------------------------------------------------------------------ */
 
 #define STRUCT_newid         ((knh_struct_t)-1)
@@ -499,7 +546,9 @@ char *konoha_getStructName(Ctx *ctx, knh_struct_t sid);
 #define KNH_TSTRUCT_SIZE 256
 #endif
 
+typedef void (*knh_fgchook)(Ctx *ctx);
 typedef void (*knh_ftraverse)(Ctx *ctx, Object *);
+
 typedef knh_uint_t                knh_hcode_t;  /* knh_hcode_t */
 #define knh_hcode_join(s1,s2)	   ((knh_hcode_t)s1 << (sizeof(knh_short_t)*8)) + s2;
 
@@ -604,9 +653,12 @@ typedef int knh_mutex_t;
 typedef int (*knh_fmutexlock)(knh_mutex_t*);
 typedef int (*knh_fmutexunlock)(knh_mutex_t*);
 
-#define KNH_LOCK(ctx, lock)  (ctx->share)->flock(lock);
+#define KNH_LOCK(ctx, lock)    (ctx->share)->flock(lock);
 #define KNH_UNLOCK(ctx, lock)  (ctx->share)->funlock(lock);
 
+#define knh_Object_sweep       (ctx)->fsweep
+#define IS_SWEEP(f)            (f == knh_Object_sweep)
+
 /* ------------------------------------------------------------------------ */
 
 typedef struct knh_ContextShare_t {
@@ -634,7 +686,7 @@ typedef struct knh_Context_t {
 
 	/* stack */
 	knh_sfp_t*                   stack;
-	knh_sfp_t*                   ebp;
+	knh_sfp_t*                   esp;
 	size_t                       stacksize;
 
 	/* root table */
@@ -646,7 +698,8 @@ typedef struct knh_Context_t {
 	knh_tStruct_t  *tStruct;
 	knh_tClass_t   *tClass;
 	knh_tExpt_t    *tExpt;
-//	struct knh_DictSet_t  *texptNameDictSet;
+	knh_ftraverse   fsweep;
+	knh_fgchook     fgchook;
 
 	/* system const */
 	knh_Object_t   *constNull;
@@ -715,7 +768,7 @@ typedef struct {
 	size_t pos;
 	struct knh_Bytes_t          *ba;
 	struct knh_OutputStream_t   *w;
-} knh_wbuf_t;
+} knh_cwb_t;
 
 /* ------------------------------------------------------------------------ */
 /* Functions */
@@ -761,13 +814,13 @@ typedef struct {
 /* KNH_DRVAPI_TYPE__BYTECONV */
 
 struct knh_BytesConv_t;
-typedef size_t (*f_bconv)(Ctx *ctx, struct knh_BytesConv_t *o, knh_bytes_t t, struct knh_Bytes_t *ba);
+typedef size_t (*knh_fbcnv)(Ctx *ctx, struct knh_BytesConv_t *o, knh_bytes_t t, struct knh_Bytes_t *ba);
 
 typedef struct {
 	int  type;
 	char *name;
-	f_bconv      fbconv;
-	f_bconv      fbconv_inverse;
+	knh_fbcnv  fbconv;
+	knh_fbcnv  fbconv_inverse;
 } knh_bconv_driapi_t;
 
 /* ------------------------------------------------------------------------ */
