@@ -132,17 +132,17 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 
 #define KLR_INITCODE(ctx, n) { \
 		knh_code_thread(ctx, pc, OPJUMP); \
-		((knh_codejmp_t*)pc)->opcode = OPCODE_SETEBP;\
+		((knh_kode_t*)pc)->opcode = OPCODE_SETESP;\
 		return; \
 	}\
 
-#define KLR_SETEBP(ctx, n) \
-	((Context*)ctx)->ebp = &(sfp[n]);\
-	if(unlikely((ctx)->stacksize - ((ctx)->ebp - (ctx)->stack) < KNH_LOCALSIZE)) {\
+#define KLR_SETESP(ctx, n) \
+	((Context*)ctx)->esp = &(sfp[n]);\
+	if(unlikely((ctx)->stacksize - ((ctx)->esp - (ctx)->stack) < KNH_LOCALSIZE)) {\
 		KNH_THROWs(ctx, "StackOverflow!!"); \
 	}\
 
-#define KLR_CHECKEBP(ctx, n) \
+#define KLR_CHECKESP(ctx, n) \
 	if(unlikely((ctx)->stacksize - (&sfp[n] - (ctx)->stack) < KNH_LOCALSIZE)) {\
 		KNH_THROWs(ctx, "StackOverflow!!"); \
 	}\
@@ -227,68 +227,92 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 
 /* ======================================================================== */
 
+#define KLR_RECSFP(ctx, sfp)
+#define KLR_RECSFP2(ctx, sfp)  ctx->esp[0].sp = sfp
+
 #define KLR_SCALL(ctx, n, shift, m) { \
 		KLR_MOV(ctx, sfp[n].o, m); \
-		sfp[n].pc = pc; \
-		((Context*)ctx)->ebp = &(sfp[n + shift]); \
-		ctx->ebp[0].pc = (sfp[n].mtd)->pc_start; \
+		((Context*)ctx)->esp = &(sfp[n + shift]); \
+		sfp[-1].pc = pc; \
+		sfp[n].pc = (sfp[n].mtd)->pc_start; \
 		(sfp[n].mtd)->fcall_1(ctx, sfp + n + 1); \
-		((Context*)ctx)->ebp = &(sfp[n]); \
+		((Context*)ctx)->esp = &(sfp[n]); \
+	} \
+
+#define KNH_SCALL(ctx, lsfp, n, m, args) { \
+		KLR_MOV(ctx, lsfp[n].o, m); \
+		((Context*)ctx)->esp = &(lsfp[n+args+2]); \
+		lsfp[-1].pc = NULL; \
+		lsfp[n].pc = (lsfp[n].mtd)->pc_start; \
+		(lsfp[n].mtd)->fcall_1(ctx, lsfp + (n + 1)); \
+		((Context*)ctx)->esp = &(lsfp[n]); \
 	} \
 
 #define KLR_FCALL(ctx, n, shift, self, m) { \
 		int n1_ = n + 1;\
 		KLR_MOVa(ctx, n1_, self); \
 		KLR_MOV(ctx, sfp[n].o, m); \
-		sfp[n].pc = pc; \
-		((Context*)ctx)->ebp = &(sfp[n + shift]); \
-		ctx->ebp[0].pc = (sfp[n].mtd)->pc_start; \
+		((Context*)ctx)->esp = &(sfp[n + shift]); \
+		sfp[-1].pc = pc; \
+		sfp[n].pc = (sfp[n].mtd)->pc_start; \
 		(sfp[n].mtd)->fcall_1(ctx, sfp + n1_); \
-		((Context*)ctx)->ebp = &(sfp[n]); \
+		((Context*)ctx)->esp = &(sfp[n]); \
 	} \
 
-
 #define KLR_CALL(ctx, n, shift, mn) { \
 		KLR_MOV(ctx, sfp[n].o, knh_Method_ufind(ctx, knh_Object_cid(sfp[n+1].o), mn)); \
-		sfp[n].pc = pc; \
-		((Context*)ctx)->ebp = &(sfp[n + shift]); \
-		ctx->ebp[0].pc = (sfp[n].mtd)->pc_start; \
+		DBG2_ASSERT(IS_Method(sfp[n].o)); \
+		((Context*)ctx)->esp = &(sfp[n + shift]); \
+		sfp[n].pc = (sfp[n].mtd)->pc_start; \
+		sfp[-1].pc = pc; \
 		(sfp[n].mtd)->fcall_1(ctx, sfp + n + 1); \
-		((Context*)ctx)->ebp = &(sfp[n]); \
+		((Context*)ctx)->esp = &(sfp[n]); \
 	} \
 
 #define KLR_ACALL(ctx, n, shift, mn, reqt) { \
 		Method *mtd_ = knh_Method_ufind(ctx, knh_Object_cid(sfp[n+1].o), mn);\
 		KLR_MOV(ctx, sfp[n].o, mtd_); \
-		knh_dynamiccall(ctx, sfp + n); \
-		sfp[n].pc = pc; \
-		((Context*)ctx)->ebp = &(sfp[n + shift]); \
-		ctx->ebp[0].pc = (sfp[n].mtd)->pc_start; \
+		((Context*)ctx)->esp = &(sfp[n + shift]); \
+		knh_sfp_typecheck(ctx, sfp + n + 1, reqt, mtd_, pc); \
+		sfp[-1].pc = pc; \
+		sfp[n].pc = (sfp[n].mtd)->pc_start; \
+		(mtd_)->fcall_1(ctx, sfp + n + 1); \
+		((Context*)ctx)->esp = &(sfp[n]); \
+		TODO(/* AMAP */); \
+	} \
+
+/* this is used only for Closure.invoke */
+
+#define KLR_ASCALL(ctx, n, shift, reqt, m) { \
+		KLR_MOV(ctx, sfp[n].o, m); \
+		((Context*)ctx)->esp = &(sfp[n + shift]); \
+		knh_sfp_typecheck(ctx, sfp + n + 1, reqt, m, pc); \
+		sfp[-1].pc = pc; \
+		sfp[n].pc = (sfp[n].mtd)->pc_start; \
 		(sfp[n].mtd)->fcall_1(ctx, sfp + n + 1); \
-		((Context*)ctx)->ebp = &(sfp[n]); \
-		TODO();\
+		((Context*)ctx)->esp = &(sfp[n]); \
 	} \
 
 #define KLR_NEW(ctx, n, flag, cid, shift, m) { \
 		KLR_MOV(ctx, sfp[n].o, m); \
 		KLR_MOV(ctx, sfp[n+1].o, new_Object_init(ctx, flag, cid, 0)); \
-		sfp[n].pc = pc; \
-		((Context*)ctx)->ebp = &(sfp[n + shift]); \
-		ctx->ebp[0].pc = (sfp[n].mtd)->pc_start; \
+		((Context*)ctx)->esp = &(sfp[n + shift]); \
+		sfp[-1].pc = pc; \
+		sfp[n].pc = (sfp[n].mtd)->pc_start; \
 		(sfp[n].mtd)->fcall_1(ctx, sfp + n + 1); \
-		((Context*)ctx)->ebp = &(sfp[n]); \
+		((Context*)ctx)->esp = &(sfp[n]); \
 	} \
 
 /* ------------------------------------------------------------------------- */
 
 #define KLR_TOSTRf(ctx, n, mn, fmt) { \
 		KLR_SWAP(ctx, n, n+1); \
-		knh_wbuf_t cwb = knh_Context_wbuf(ctx);\
+		knh_cwb_t cwb = new_cwb(ctx);\
 		KLR_MOV(ctx, sfp[n+2].o, cwb.w);\
 		KLR_MOV(ctx, sfp[n+3].o, fmt);\
 		Method *mtd_ = knh_tMethod_findMT(ctx, knh_Object_cid(sfp[n+1].o), mn);\
 		KLR_SCALL(ctx, n, 4, mtd_);\
-		KLR_MOV(ctx, sfp[n].o, new_String__wbuf(ctx, cwb)); \
+		KLR_MOV(ctx, sfp[n].o, new_String__cwb(ctx, cwb)); \
 	}\
 
 #define KLR_TOSTR(ctx, n, mn)  KLR_TOSTRf(ctx, n, mn, KNH_NULL)
@@ -432,6 +456,24 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------- */
 
+#define NPC  /* for KNH_TRY */
+
+#define KNH_SETJUMP(hdlr) setjmp(DP(hdlr)->jmpbuf)
+
+#define KNH_TRY(ctx, JUMP, lsfp, hn)  {\
+		ExceptionHandler* _hdr = lsfp[hn].hdr; \
+		if(!knh_ExceptionHandler_isJumpable(_hdr)) { \
+			knh_ExceptionHandler_setJumpable(_hdr, 1); \
+			int jump = KNH_SETJUMP(_hdr); \
+			if(jump != 0) { \
+				((Context*)ctx)->esp = &(lsfp[hn]); \
+				KLR_JMP(ctx, NPC, JUMP); \
+			} \
+			DP(_hdr)->esp = ctx->esp; \
+		} \
+		knh_ExceptionHandler_setCatching(_hdr, 1); \
+	} \
+
 #define KLR_TRY(ctx, PC, JUMP, hn)  {\
 		ExceptionHandler* _hdr = sfp[hn].hdr; \
 		if(!IS_ExceptionHandler(_hdr)) { \
@@ -442,10 +484,12 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 			knh_ExceptionHandler_setJumpable(_hdr, 1); \
 			int jump = KNH_SETJUMP(_hdr); \
 			if(jump != 0) { \
+				((Context*)ctx)->esp = DP(_hdr)->esp; \
 				pc = DP(_hdr)->pc; \
 				KLR_JMP(ctx, PC, JUMP); \
 			} \
-			DP(_hdr)->pc = pc; \
+			DP(_hdr)->esp = ctx->esp; \
+			DP(_hdr)->pc  = pc; \
 		} \
 		knh_ExceptionHandler_setCatching(_hdr, 1); \
 	} \
@@ -597,42 +641,20 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 /* ------------------------------------------------------------------------ */
 
 
-#define KNH_LOCAL(ctx)  ((Context*)ctx)->ebp
-#define KNH_LOCALBACK(ctx, lsfp)  ((Context*)ctx)->ebp = lsfp
+#define KNH_LOCAL(ctx)  ((Context*)ctx)->esp
+#define KNH_LOCALBACK(ctx, lsfp)  ((Context*)ctx)->esp = lsfp
 
 #define KNH_LPUSH(ctx, v)  {\
-		KLR_MOV(ctx, ((Context*)ctx)->ebp[0].o, v); \
-		((Context*)ctx)->ebp += 1;\
+		KLR_MOV(ctx, ((Context*)ctx)->esp[0].o, v); \
+		((Context*)ctx)->esp += 1;\
 	}
 
-#define KNH_SCALL(ctx, lsfp, n, m, args) { \
-		KLR_MOV(ctx, lsfp[n].o, m); \
-		((Context*)ctx)->ebp = &(lsfp[n+args+2]); \
-		ctx->ebp[0].pc = (lsfp[n].mtd)->pc_start; \
-		(lsfp[n].mtd)->fcall_1(ctx, lsfp + (n + 1)); \
-		((Context*)ctx)->ebp = &(lsfp[n]); \
-	} \
-
 #define KNH_SMAP(ctx, lsfp, n, m)  { \
 		KLR_MOV(ctx, lsfp[n+1].o, m);\
 		KNH_ASSERT(IS_Mapper(m)); \
 		(lsfp[n+1].mpr)->fmap_1(ctx, lsfp + n); \
 	} \
 
-#define NPC  /* for KNH_TRY */
-
-#define KNH_TRY(ctx, JUMP, lsfp, hn)  {\
-		ExceptionHandler* _hdr = lsfp[hn].hdr; \
-		if(!knh_ExceptionHandler_isJumpable(_hdr)) { \
-			knh_ExceptionHandler_setJumpable(_hdr, 1); \
-			int jump = KNH_SETJUMP(_hdr); \
-			if(jump != 0) { \
-				((Context*)ctx)->ebp = &(lsfp[hn]); \
-				KLR_JMP(ctx, NPC, JUMP); \
-			} \
-		} \
-		knh_ExceptionHandler_setCatching(_hdr, 1); \
-	} \
 
 #define KNH_PRINT_STACKTRACE(ctx, lsfp, hn) {\
 		KNH_ASSERT(IS_ExceptionHandler(lsfp[hn].hdr));\
