@@ -42,44 +42,44 @@ extern "C" {
 static
 size_t bconv__toEncodedURL(Ctx *ctx, BytesConv *o, knh_bytes_t in, knh_Bytes_t *out)
 {
-   size_t size = in.len;
-   size_t i;
-   for(i = 0; i < size; i++){
-      char c = in.buf[i];
-      if(c == ' '){
-	 knh_Bytes_putc(ctx, out, '+');
-      }else if(('A' <= c && c <= 'Z') ||
-	       ('a' <= c && c <= 'z') ||
-	       ('0' <= c && c <= '9') ||
-	       (c == '@') ||
-	       (c == '*') ||
-	       (c == '-') ||
-	       (c == '.') ||
-	       (c == '_') ||
-	       (c == '(') ||
-	       (c == ')') ||
-	       (c == '!') ||
-	       (c == ';') ||
-	       (c == '/')){
-	 knh_Bytes_putc(ctx, out, c);
-      }else{
-	 knh_Bytes_putc(ctx, out, '%');
-	 int j;
-	 for(j = 0; j < 2; j++){
-	    int a = ((0x0f << 4) & (c << (4 * j))) >> 4;
-	    int b;
-	    if(0x0a <= a && a <= 0x0f){
-	       b = a - 0x0a + 'a';
-	    }else if(0 <= a && a <= 9){
-	       b = a + '0';
-	    }else{
-	       return -1;
-	    }
-	    knh_Bytes_putc(ctx, out, b);
-	 }
-      }
-   }
-   return in.len;
+	size_t size = in.len;
+	size_t i;
+	for(i = 0; i < size; i++){
+		char c = in.buf[i];
+		if(c == ' '){
+			knh_Bytes_putc(ctx, out, '+');
+		}else if(('A' <= c && c <= 'Z') ||
+				('a' <= c && c <= 'z') ||
+				('0' <= c && c <= '9') ||
+				(c == '@') ||
+				(c == '*') ||
+				(c == '-') ||
+				(c == '.') ||
+				(c == '_') ||
+				(c == '(') ||
+				(c == ')') ||
+				(c == '!') ||
+				(c == ';') ||
+				(c == '/')){
+			knh_Bytes_putc(ctx, out, c);
+		}else{
+			knh_Bytes_putc(ctx, out, '%');
+			int j;
+			for(j = 0; j < 2; j++){
+				int a = ((0x0f << 4) & (c << (4 * j))) >> 4;
+				int b;
+				if(0x0a <= a && a <= 0x0f){
+					b = a - 0x0a + 'a';
+				}else if(0 <= a && a <= 9){
+					b = a + '0';
+				}else{
+					return -1;
+				}
+				knh_Bytes_putc(ctx, out, b);
+			}
+		}
+	}
+	return in.len;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -88,7 +88,7 @@ size_t bconv__toEncodedURL(Ctx *ctx, BytesConv *o, knh_bytes_t in, knh_Bytes_t *
 static
 METHOD knh__String_toEncodedURL(Ctx *ctx, knh_sfp_t *sfp)
 {
-   KNH_RETURN(ctx, sfp, new_String_bconv(ctx, sfp[0].s, bconv__toEncodedURL));
+   KNH_RETURN(ctx, sfp, new_String__fbcnv(ctx, sfp[0].s, bconv__toEncodedURL, (BytesConv*)KNH_NULL));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -96,46 +96,46 @@ METHOD knh__String_toEncodedURL(Ctx *ctx, knh_sfp_t *sfp)
 static
 size_t bconv__toDecodedURL(Ctx *ctx, BytesConv *o, knh_bytes_t in, knh_Bytes_t *out)
 {
-   size_t i, size = in.len;
-   for(i=0; i < size; i++) {
-      char c = in.buf[i];
-      if(c == '+'){
-	 knh_Bytes_putc(ctx, out, ' ');
-      }else if(('A' <= c && c <= 'Z') ||
-	       ('a' <= c && c <= 'z') ||
-	       ('0' <= c && c <= '9') ||
-	       (c == '@') ||
-	       (c == '*') ||
-	       (c == '-') ||
-	       (c == '.') ||
-	       (c == '_') ||
-	       (c == '(') ||
-	       (c == ')') ||
-	       (c == '!') ||
-	       (c == ';') ||
-	       (c == '/')){
-	 knh_Bytes_putc(ctx, out, c);
-      }else if(c == '%'){
-	 int j;
-	 int a = 0;
-	 for(j=0; j < 2; j++){
-	    c = in.buf[++i];
-	    int b;
-	    if('0' <= c && c <= '9'){
-	       b = c - '0';
-	    }else if('a' <= c && c <= 'f'){
-	       b = c - 'a' + 0x0a;
-	    }else if('A' <= c && c <= 'F'){
-	       b = c - 'A' + 0x0a;
-	    }else{
-	       return -1;
-	    }
-	    a = (a << 4) + b;
-	 }
-	 knh_Bytes_putc(ctx, out, a);
-      }
-   }
-   return in.len;
+	size_t i, size = in.len;
+	for(i=0; i < size; i++) {
+		char c = in.buf[i];
+		if(c == '+'){
+			knh_Bytes_putc(ctx, out, ' ');
+		}else if(('A' <= c && c <= 'Z') ||
+				('a' <= c && c <= 'z') ||
+				('0' <= c && c <= '9') ||
+				(c == '@') ||
+				(c == '*') ||
+				(c == '-') ||
+				(c == '.') ||
+				(c == '_') ||
+				(c == '(') ||
+				(c == ')') ||
+				(c == '!') ||
+				(c == ';') ||
+				(c == '/')){
+			knh_Bytes_putc(ctx, out, c);
+		}else if(c == '%'){
+			int j;
+			int a = 0;
+			for(j=0; j < 2; j++){
+				c = in.buf[++i];
+				int b;
+				if('0' <= c && c <= '9'){
+					b = c - '0';
+				}else if('a' <= c && c <= 'f'){
+					b = c - 'a' + 0x0a;
+				}else if('A' <= c && c <= 'F'){
+					b = c - 'A' + 0x0a;
+				}else{
+					return -1;
+				}
+				a = (a << 4) + b;
+			}
+			knh_Bytes_putc(ctx, out, a);
+		}
+	}
+	return in.len;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -144,7 +144,7 @@ size_t bconv__toDecodedURL(Ctx *ctx, BytesConv *o, knh_bytes_t in, knh_Bytes_t *
 static
 METHOD knh__String_toDecodedURL(Ctx *ctx, knh_sfp_t *sfp)
 {
-   KNH_RETURN(ctx, sfp, new_String_bconv(ctx, sfp[0].s, bconv__toDecodedURL));
+   KNH_RETURN(ctx, sfp, new_String__fbcnv(ctx, sfp[0].s, bconv__toDecodedURL, (BytesConv*)KNH_NULL));
 }
 
 /* ======================================================================== */
@@ -160,9 +160,9 @@ KNHAPI(int) knhcgi_init(Ctx *ctx)
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knhcgi_traverse(Ctx *ctx, f_traverse ftr)
+KNHAPI(void) knhcgi_traverse(Ctx *ctx, knh_ftraverse ftr)
 {
-   if(konoha_issweep(ftr)) {
+   if(IS_SWEEP(ftr)) {
 
    }
 }
@@ -184,7 +184,7 @@ KNHAPI(knh_pkgmeta_t*) knhcgi_load(Ctx *ctx)
 {
    return &(PKGDATA);
 }
-   
+
 /* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
