@@ -48,7 +48,7 @@ int knhnaruto_init(Ctx *ctx)
 
 void knhnaruto_traverse(Ctx *ctx, knh_ftraverse ftr)
 {
-	if(konoha_issweep(ftr)) {
+	if(IS_SWEEP(ftr)) {
 
 	}
 }
