@@ -272,7 +272,7 @@ int knhunix_init(Ctx *ctx)
 
 void knhunix_traverse(Ctx *ctx, knh_ftraverse ftr)
 {
-	if(konoha_issweep(ftr)) {
+	if(IS_SWEEP(ftr)) {
 
 	}
 }
