@@ -184,7 +184,7 @@ METHOD knh__Object_opIs(Ctx *ctx, knh_sfp_t *sfp)
 //		knh_sfp_t *lsfp = KNH_LOCAL(ctx);
 //		KNH_LPUSH(ctx, sfp[1].o);
 //		VM_MAP(ctx, scid);
-//		KNH_SETv(ctx, sfp[1].o, ctx->ebp[0].o);
+//		KNH_SETv(ctx, sfp[1].o, ctx->esp[0].o);
 //		KNH_LOCALBACK(ctx, lsfp);
 //		if(IS_NULL(sfp[1].o)) {
 //			KNH_RETURN_Boolean(ctx, sfp, 0);
@@ -371,12 +371,12 @@ MAPPER knh_Object_String(Ctx *ctx, knh_sfp_t *sfp)
 	if(IS_NULL(mtd)) {
 		KNH_MAPPED(ctx, sfp, TS_EMPTY);
 	}
-	knh_wbuf_t cwb = knh_Context_wbuf(ctx);
+	knh_cwb_t cwb = new_cwb(ctx);
 	KNH_MOV(ctx, sfp[2].o, o);
 	KNH_MOV(ctx, sfp[3].o, cwb.w);
 	KNH_MOV(ctx, sfp[4].o, KNH_NULL);
 	KNH_SCALL(ctx, sfp, 1, mtd, 2);
-	KNH_MAPPED(ctx, sfp, new_String__wbuf(ctx, cwb));
+	KNH_MAPPED(ctx, sfp, new_String__cwb(ctx, cwb));
 }
 
 /* ======================================================================== */
@@ -411,7 +411,7 @@ void knh_Nue__k(Ctx *ctx, Nue *b, OutputStream *w, String *m)
 
 void knh_Class__s(Ctx *ctx, Class *o, OutputStream *w, String *m)
 {
-	knh_write__class(ctx, w, o->cid);
+	knh_write_cid(ctx, w, o->cid);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -419,7 +419,7 @@ void knh_Class__s(Ctx *ctx, Class *o, OutputStream *w, String *m)
 
 void knh_Class__k(Ctx *ctx, Class *o, OutputStream *w, String *m)
 {
-	knh_write__type(ctx, w, o->type);
+	knh_write_type(ctx, w, o->type);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -531,7 +531,7 @@ void knh_Method__man(Ctx *ctx, Method *o, OutputStream *w, knh_class_t cid)
 	if(knh_Method_rztype(o) == TYPE_void) {
 		knh_write(ctx, w, knh_String_tobytes(TS_void));
 	}else{
-		knh_write__type(ctx, w, knh_pmztype_totype(ctx, knh_Method_rztype(o), cid));
+		knh_write_type(ctx, w, knh_pmztype_totype(ctx, knh_Method_rztype(o), cid));
 	}
 	knh_putc(ctx, w, ' ');
 
@@ -539,7 +539,7 @@ void knh_Method__man(Ctx *ctx, Method *o, OutputStream *w, knh_class_t cid)
 		knh_write__s(ctx, w, CTXCLASSN(cid));
 		knh_putc(ctx, w, '.');
 	}
-	knh_write__mn(ctx, w, DP(o)->mn);
+	knh_write_mn(ctx, w, DP(o)->mn);
 
 	knh_putc(ctx, w, '(');
 	size_t i;
@@ -548,7 +548,7 @@ void knh_Method__man(Ctx *ctx, Method *o, OutputStream *w, knh_class_t cid)
 			knh_write_delim(ctx, w);
 		}
 		knh_mfield_t mf = knh_Method_pfield(o, i);
-		knh_write__type(ctx, w, knh_pmztype_totype(ctx, mf.type, cid));
+		knh_write_type(ctx, w, knh_pmztype_totype(ctx, mf.type, cid));
 		knh_putc(ctx, w, ' ');
 		knh_write(ctx, w, B(FIELDN(mf.fn)));
 	}
@@ -693,7 +693,7 @@ void knh_Method__s(Ctx *ctx, Method *o, OutputStream *w, String *m)
 {
 	knh_write__s(ctx, w, CTXCLASSN(DP(o)->cid));
 	knh_putc(ctx, w, '.');
-	knh_write__mn(ctx, w, DP(o)->mn);
+	knh_write_mn(ctx, w, DP(o)->mn);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -710,7 +710,7 @@ void knh_Method__k(Ctx *ctx, Method *o, OutputStream *w, String *m)
 		knh_write(ctx, w, knh_String_tobytes(TS_void));
 	}else{
 //		knh_write__type(ctx, w, knh_pmztype_totype(ctx, knh_Method_rtype(o), DP(o)->cid));
-		knh_write__type(ctx, w, knh_Method_rztype(o));
+		knh_write_type(ctx, w, knh_Method_rztype(o));
 	}
 	knh_putc(ctx, w, ' ');
 	knh_Method__s(ctx, o, w, (String*)KNH_NULL);
@@ -722,7 +722,7 @@ void knh_Method__k(Ctx *ctx, Method *o, OutputStream *w, String *m)
 		}
 		knh_mfield_t mf = knh_Method_pfield(o, i);
 		//knh_write__type(ctx, w, knh_pmztype_totype(ctx, mf.type, DP(o)->cid));
-		knh_write__type(ctx, w, mf.type);
+		knh_write_type(ctx, w, mf.type);
 		knh_putc(ctx, w, ' ');
 		knh_write(ctx, w, B(FIELDN(mf.fn)));
 	}
