@@ -128,10 +128,10 @@ METHOD knh__String_opSize(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__String_opAdd(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_wbuf_t cwb = knh_Context_wbuf(ctx);
+	knh_cwb_t cwb = new_cwb(ctx);
 	knh_sfp_format(ctx, sfp, METHODN__s, cwb.w, KNH_NULL);
 	knh_sfp_format(ctx, sfp+1, METHODN__s, cwb.w, KNH_NULL);
-	KNH_RETURN(ctx, sfp, new_String__wbuf(ctx, cwb));
+	KNH_RETURN(ctx, sfp, new_String__cwb(ctx, cwb));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -142,12 +142,12 @@ METHOD knh__String_concat(Ctx *ctx, knh_sfp_t *sfp)
 	knh_sfp_t *v = sfp + 1;
 	int ac = knh_sfp_argc(ctx, v);
 	size_t i;
-	knh_wbuf_t cwb = knh_Context_wbuf(ctx);
+	knh_cwb_t cwb = new_cwb(ctx);
 	knh_sfp_format(ctx, sfp, METHODN__s, cwb.w, KNH_NULL);
 	for(i = 0; i < ac; i++) {
 		knh_sfp_format(ctx, v + i, METHODN__s, cwb.w, KNH_NULL);
 	}
-	KNH_RETURN(ctx, sfp, new_String__wbuf(ctx, cwb));
+	KNH_RETURN(ctx, sfp, new_String__cwb(ctx, cwb));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -158,7 +158,7 @@ METHOD knh__String_opSub(Ctx *ctx, knh_sfp_t *sfp)
 	knh_bytes_t base = knh_String_tobytes(sfp[0].s);
 	knh_bytes_t t = knh_String_tobytes(sfp[1].s);
 	knh_uchar_t c = t.buf[0];
-	knh_wbuf_t wbuf = knh_Context_wbuf(ctx);
+	knh_cwb_t cwb = new_cwb(ctx);
 	size_t i;
 	for(i = 0; i < base.len; i++) {
 		if(base.buf[i] == c) {
@@ -171,14 +171,14 @@ METHOD knh__String_opSub(Ctx *ctx, knh_sfp_t *sfp)
 				continue;
 			}
 		}
-		knh_Bytes_putc(ctx, wbuf.ba, base.buf[i]);
+		knh_Bytes_putc(ctx, cwb.ba, base.buf[i]);
 	}
-	if(base.len == knh_wbuf_size(wbuf)) {
-		knh_wbuf_clear(wbuf);
+	if(base.len == knh_cwb_size(cwb)) {
+		knh_cwb_clear(cwb);
 		KNH_RETURN(ctx, sfp, sfp[0].o);
 	}
 	else {
-		KNH_RETURN(ctx, sfp, new_String__wbuf(ctx, wbuf));
+		KNH_RETURN(ctx, sfp, new_String__cwb(ctx, cwb));
 	}
 }
 
@@ -204,17 +204,17 @@ METHOD knh__String_replace(Ctx *ctx, knh_sfp_t *sfp)
 	knh_bytes_t target = knh_String_tobytes(sfp[1].s);
 	knh_bytes_t alt = knh_String_tobytes(sfp[2].s);
 
-	knh_wbuf_t wbuf = knh_Context_wbuf(ctx);
+	knh_cwb_t cwb = new_cwb(ctx);
 
 	size_t i;
 	int s = 0, ch = target.buf[0];
 	for(i = 0; i < base.len - target.len+1; i++) {
 		if(base.buf[i] == ch && knh_bytes_equals_(base, i, target)) {
-			knh_Bytes_write(ctx, wbuf.ba, alt);
+			knh_Bytes_write(ctx, cwb.ba, alt);
 			i += target.len - 1;
 			s = i;
 		}else {
-			knh_Bytes_putc(ctx, wbuf.ba, base.buf[i]);
+			knh_Bytes_putc(ctx, cwb.ba, base.buf[i]);
 		}
 	}
 	if(s == 0) {
@@ -222,8 +222,8 @@ METHOD knh__String_replace(Ctx *ctx, knh_sfp_t *sfp)
 	}
 	else {
 		knh_bytes_t hs = {base.buf + i, base.len - i};
-		knh_Bytes_write(ctx, wbuf.ba, hs);
-		KNH_RETURN(ctx, sfp, new_String__wbuf(ctx, wbuf));
+		knh_Bytes_write(ctx, cwb.ba, hs);
+		KNH_RETURN(ctx, sfp, new_String__cwb(ctx, cwb));
 	}
 }
 
@@ -240,12 +240,12 @@ METHOD knh__String_opMul(Ctx *ctx, knh_sfp_t *sfp)
 	}
 	else {
 		knh_bytes_t base = knh_String_tobytes(sfp[0].s);
-		knh_wbuf_t wbuf = knh_Context_wbuf(ctx);
+		knh_cwb_t cwb = new_cwb(ctx);
 		knh_int_t i;
 		for(i = 0; i < n; i++) {
-			knh_Bytes_write(ctx, wbuf.ba, base);
+			knh_Bytes_write(ctx, cwb.ba, base);
 		}
-		KNH_RETURN(ctx, sfp, new_String__wbuf(ctx, wbuf));
+		KNH_RETURN(ctx, sfp, new_String__cwb(ctx, cwb));
 	}
 }
 
@@ -254,7 +254,7 @@ METHOD knh__String_opMul(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__String_opFmt(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_wbuf_t cwb = knh_Context_wbuf(ctx);
+	knh_cwb_t cwb = new_cwb(ctx);
 	knh_sfp_t *v = sfp + 1;
 	int ac = knh_sfp_argc(ctx, v);
 	knh_bytes_t fmt = knh_String_tobytes(sfp[0].s);
@@ -319,7 +319,7 @@ METHOD knh__String_opFmt(Ctx *ctx, knh_sfp_t *sfp)
 		knh_Bytes_write(ctx, cwb.ba, fmt);
 	}
 	L_ERROR:;
-	KNH_RETURN(ctx, sfp, new_StringX__wbuf(ctx, knh_Object_cid(sfp[0].o), cwb));
+	KNH_RETURN(ctx, sfp, new_StringX__cwb(ctx, knh_Object_cid(sfp[0].o), cwb));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -430,22 +430,6 @@ METHOD knh__String_opOffset(Ctx *ctx, knh_sfp_t *sfp)
 
 /* ======================================================================== */
 
-KNHAPI(String*) new_String_bconv(Ctx *ctx, String *s, f_bconv bconv)
-{
-	knh_bytes_t base = knh_String_tobytes(s);
-	Bytes *ba = knh_Context_openBConvBuf(ctx);
-	bconv(ctx, (BytesConv*)KNH_NULL, base, ba);
-	if(knh_strncmp((char*)(base.buf), (char*)(ba->buf), base.len) == 0) {
-		knh_Context_closeBConvBuf(ctx, ba);
-		return s;
-	}
-	else {
-		s = new_String(ctx, knh_Bytes_tobytes(ba), NULL);
-		knh_Context_closeBConvBuf(ctx, ba);
-		return s;
-	}
-}
-
 /* ------------------------------------------------------------------------ */
 /* @func String:+ */
 
@@ -468,7 +452,7 @@ size_t bconv__toLower(Ctx *ctx, BytesConv *o, knh_bytes_t t, knh_Bytes_t *ba)
 
 METHOD knh__String_toLower(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN(ctx, sfp, new_String_bconv(ctx, sfp[0].s, bconv__toLower));
+	KNH_RETURN(ctx, sfp, new_String__fbcnv(ctx, sfp[0].s, bconv__toLower, (BytesConv*)KNH_NULL));
 }
 
 /* ------------------------------------------------------------------------ */
@@ -493,7 +477,7 @@ size_t bconv__toUpper(Ctx *ctx, BytesConv *o, knh_bytes_t t, knh_Bytes_t *ba)
 
 METHOD knh__String_toUpper(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KNH_RETURN(ctx, sfp, new_String_bconv(ctx, sfp[0].s, bconv__toUpper));
+	KNH_RETURN(ctx, sfp, new_String__fbcnv(ctx, sfp[0].s, bconv__toUpper, (BytesConv*)KNH_NULL));
 }
 
 /* ------------------------------------------------------------------------ */
