@@ -194,6 +194,7 @@ void knh_System__dump(Ctx *ctx, System *o, OutputStream *w, String *m)
 	knh_write__s(ctx, w, " thread");
 #endif
 	knh_write_USING_REGEX(ctx, w);
+	knh_printf(ctx, w, " used_memory:%d kb", (int)ctx->share->statUsedMemorySize / 1024);
 	knh_write_EOL(ctx, w);
 }
 
@@ -335,7 +336,7 @@ METHOD knh__Context_setErr(Ctx *ctx, knh_sfp_t *sfp)
 void knh_Context__dump(Ctx *ctx, Context *b, OutputStream *w, String *m)
 {
 //	knh_int_t i = 0;
-//	for(i = b->ebp - b->stack; i >= 0; i--) {
+//	for(i = b->esp - b->stack; i >= 0; i--) {
 //		knh_printf(ctx, w, "[%d]\t%o\n", i, b->stack[i]);
 //	}
 }
