@@ -42,12 +42,11 @@ extern "C" {
 
 #define _KNH_FLAG_TF_NN                KNH_FLAG_T0
 #define _TYPE_ISNULLABLE(t)            ((t & KNH_FLAG_TF_NN)==0)
-#define _IS_NNTYPE(t)             ((t & KNH_FLAG_TF_NN)==KNH_FLAG_TF_NN)
-#define _NNTYPE_cid(c)             (c|KNH_FLAG_TF_NN)
+#define _IS_NNTYPE(t)                  ((t & KNH_FLAG_TF_NN)==KNH_FLAG_TF_NN)
+#define _NNTYPE_cid(c)                 (c|KNH_FLAG_TF_NN)
 #define _TYPE_TONNTYPE(t)              (t|KNH_FLAG_TF_NN)
 
-#define _TYPE_UNMASK(t)                (t&(~(KNH_FLAG_TF_NN|KNH_FLAG_T1)))
-#define _CLASS_type(t)             (t&(~KNH_FLAG_TF_NN))
+#define _CLASS_type(t)                (t&(~KNH_FLAG_TF_NN))
 
 #define _TYPEQN(t)                     TYPEN(t), TYPEQ(t)
 
@@ -57,56 +56,88 @@ extern "C" {
 #define _TYPE_any                      CLASS_Any
 #define _NNTYPE_any                    CLASS_any
 
-//#define _KNH_FLAG_TF_PMZ               KNH_FLAG_T1
-//#define _TYPE_ISPMZ(t)                 ((t & KNH_FLAG_TF_PMZ)==KNH_FLAG_TF_PMZ)
-//#define _CLASS_PMZ(t)                  (t) /*((t | KNH_FLAG_TF_PMZ))*/
-//#define _TYPE_PMZ(t)                   ((t | KNH_FLAG_TF_PMZ))
-//#define _TYPE_UNMASK_PMZ(t)            (t&(~KNH_FLAG_TF_PMZ))
-
 /* ------------------------------------------------------------------------ */
 
-knh_class_t
-knh_pmztype_toclass(Ctx *ctx, knh_type_t t, knh_class_t this_cid)
+knh_class_t knh_pmztype_toclass(Ctx *ctx, knh_type_t t, knh_class_t this_cid)
 {
 	knh_class_t cid = CLASS_type(t);
+	KNH_ASSERT_cid(cid);
 	KNH_ASSERT_cid(this_cid);
 	if(cid == CLASS_This) {
 		return this_cid;
 	}
-	else if(cid == CLASS_Any1) {
-		return knh_tClass[this_cid].p1;
+	if(cid == CLASS_Any1) {
+		return ctx->tClass[this_cid].p1;
 	}
-	else if(cid == CLASS_Any2) {
-		return knh_tClass[this_cid].p2;
+	if(cid == CLASS_Any2) {
+		return ctx->tClass[this_cid].p2;
 	}
-	else if(knh_tClass_isPMZ(cid)) { /* Iterator<This> */
-		knh_class_t p1 = knh_pmztype_toclass(ctx, knh_tClass[cid].p1, this_cid);
-		knh_class_t p2 = knh_pmztype_toclass(ctx, knh_tClass[cid].p2, this_cid);
-		return knh_pmzclass(ctx, knh_tClass[cid].bcid, p1, p2);
+	if(ctx->tClass[cid].bcid == CLASS_Closure) {
+		knh_type_t r0 = ctx->tClass[cid].r0;
+		knh_type_t p1 = ctx->tClass[cid].p1;
+		knh_type_t p2 = ctx->tClass[cid].p2;
+		knh_type_t p3 = ctx->tClass[cid].p3;
+		knh_type_t rr0 = knh_pmztype_totype(ctx, r0, this_cid);
+		knh_type_t pp1 = knh_pmztype_totype(ctx, p1, this_cid);
+		knh_type_t pp2 = knh_pmztype_totype(ctx, p2, this_cid);
+		knh_type_t pp3 = knh_pmztype_totype(ctx, p3, this_cid);
+		if(r0 != rr0 || p1 != pp1 || p2 != pp2 || p3 != pp3) {
+			return knh_class_Closure(ctx, rr0, pp1, pp2, pp3);
+		}
+		return cid;
+	}
+	if(knh_Class_isGenerics(cid)) { /* Iterator<This> */
+		knh_class_t p1 = ctx->tClass[cid].p1;
+		knh_class_t p2 = ctx->tClass[cid].p2;
+		knh_class_t pp1 = knh_pmztype_toclass(ctx, p1, this_cid);
+		knh_class_t pp2 = knh_pmztype_toclass(ctx, p2, this_cid);
+		if(p1 != pp1 || p2 != pp2) {
+			return knh_class_Generics(ctx, ctx->tClass[cid].bcid, pp1, pp2);
+		}
+		return cid;
 	}
 	return cid;
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_type_t
-knh_pmztype_totype(Ctx *ctx, knh_type_t t, knh_class_t this_cid)
+knh_type_t knh_pmztype_totype(Ctx *ctx, knh_type_t t, knh_class_t this_cid)
 {
 	knh_class_t cid = CLASS_type(t), mask = (t & KNH_FLAG_TF_NN);
+	KNH_ASSERT_cid(cid);
 	KNH_ASSERT_cid(this_cid);
 	if(cid == CLASS_This) {
 		return this_cid | mask;
 	}
-	else if(cid == CLASS_Any1) {
-		return knh_tClass[this_cid].p1 | mask;
+	if(cid == CLASS_Any1) {
+		return ctx->tClass[this_cid].p1 | mask;
 	}
-	else if(cid == CLASS_Any2) {
-		return knh_tClass[this_cid].p2 | mask;
+	if(cid == CLASS_Any2) {
+		return ctx->tClass[this_cid].p2 | mask;
 	}
-	else if(knh_tClass_isPMZ(cid)) { /* Iterator<This> */
-		knh_class_t p1 = knh_pmztype_toclass(ctx, knh_tClass[cid].p1, this_cid);
-		knh_class_t p2 = knh_pmztype_toclass(ctx, knh_tClass[cid].p2, this_cid);
-		return knh_pmzclass(ctx, knh_tClass[cid].bcid, p1, p2) | mask;
+	if(ctx->tClass[cid].bcid == CLASS_Closure) {
+		knh_type_t r0 = ctx->tClass[cid].r0;
+		knh_type_t p1 = ctx->tClass[cid].p1;
+		knh_type_t p2 = ctx->tClass[cid].p2;
+		knh_type_t p3 = ctx->tClass[cid].p3;
+		knh_type_t rr0 = knh_pmztype_totype(ctx, r0, this_cid);
+		knh_type_t pp1 = knh_pmztype_totype(ctx, p1, this_cid);
+		knh_type_t pp2 = knh_pmztype_totype(ctx, p2, this_cid);
+		knh_type_t pp3 = knh_pmztype_totype(ctx, p3, this_cid);
+		if(r0 != rr0 || p1 != pp1 || p2 != pp2 || p3 != pp3) {
+			return knh_class_Closure(ctx, rr0, pp1, pp2, pp3) | mask;
+		}
+		return cid | mask;
+	}
+	if(knh_Class_isGenerics(cid)) { /* Iterator<This> */
+		knh_class_t p1 = ctx->tClass[cid].p1;
+		knh_class_t p2 = ctx->tClass[cid].p2;
+		knh_class_t pp1 = knh_pmztype_toclass(ctx, p1, this_cid);
+		knh_class_t pp2 = knh_pmztype_toclass(ctx, p2, this_cid);
+		if(p1 != pp1 || p2 != pp2) {
+			return knh_class_Generics(ctx, ctx->tClass[cid].bcid, pp1, pp2) | mask;
+		}
+		return cid | mask;
 	}
 	return t;
 }
@@ -121,8 +152,8 @@ knh_class_t knh_class_Array(Ctx *ctx, knh_class_t p1)
 	knh_snprintf(buf, sizeof(buf), "%s[]", CLASSN(p1));
 	knh_class_t cid = knh_NameSpace_getClass(ctx, knh_rootNameSpace, B(buf));
 	if(cid == CLASS_unknown) {
-		DBG2_P("Generating %s", buf);
-		return KNH_TPMZCLASS(ctx, CLASS_newid, new_String0(ctx, B(buf), NULL), CLASS_Array, p1, CLASS_Nue);
+		//DBG2_P("Generating %s", buf);
+		return konoha_addGenericsClass(ctx, CLASS_newid, new_String(ctx, B(buf), NULL), CLASS_Array, p1, CLASS_Nue);
 	}
 	return cid;
 }
@@ -136,66 +167,61 @@ knh_class_t knh_class_Iterator(Ctx *ctx, knh_class_t p1)
 	knh_snprintf(buf, sizeof(buf), "%s..", CLASSN(p1));
 	knh_class_t cid = knh_NameSpace_getClass(ctx, knh_rootNameSpace, B(buf));
 	if(cid == CLASS_unknown) {
-		DBG2_P("*** Generating %s ***", buf);
-		return KNH_TPMZCLASS(ctx, CLASS_newid, new_String0(ctx, B(buf), NULL), CLASS_Iterator, p1, CLASS_Nue);
+		//DBG2_P("*** Generating %s ***", buf);
+		return konoha_addGenericsClass(ctx, CLASS_newid, new_String(ctx, B(buf), NULL), CLASS_Iterator, p1, CLASS_Nue);
 	}
 	return cid;
 }
 
 /* ------------------------------------------------------------------------ */
 
-knh_class_t knh_pmzclass(Ctx *ctx, knh_class_t bcid, knh_class_t p1, knh_class_t p2)
+knh_class_t knh_class_Generics(Ctx *ctx, knh_class_t bcid, knh_class_t p1, knh_class_t p2)
 {
-	char buf[CLASSNAME_BUFSIZ];
+	DBG2_ASSERT(bcid != CLASS_Closure);
 	if(bcid == CLASS_Iterator) {
 		return knh_class_Iterator(ctx, p1);
 	}
 	else if(bcid == CLASS_Array){
 		return knh_class_Array(ctx, p1);
 	}
-	else {
-		knh_snprintf(buf, sizeof(buf), "%s<%s,%s>", CLASSN(bcid), CLASSN(p1), CLASSN(p2));
+	if(p1 == CLASS_Any && (p2 == CLASS_Any || p2 == CLASS_Nue)) {
+		return bcid;
 	}
-	{
+	else {
+		char buf[CLASSNAME_BUFSIZ];
+		knh_snprintf(buf, sizeof(buf), "%s<%s,%s>", CLASSN(bcid), CLASSN(p1), CLASSN(p2));
 		knh_class_t cid = knh_NameSpace_getClass(ctx, knh_rootNameSpace, B(buf));
 		if(cid == CLASS_unknown) {
 			//DBG2_P("Generating %s", buf);
-			return KNH_TPMZCLASS(ctx, CLASS_newid, new_String0(ctx, B(buf), NULL), bcid, p1, p2);
+			return konoha_addGenericsClass(ctx, CLASS_newid, new_String(ctx, B(buf), NULL), bcid, p1, p2);
 		}
 		return cid;
 	}
 }
 
-///* ------------------------------------------------------------------------ */
-///* [byte] */
-//
-//knh_bool_t knh_bytes_isPluralType(knh_bytes_t t)
-//{
-//	if(t.len > 2 && t.buf[t.len-1] == '.' && t.buf[t.len-2] == '.') {
-//		return 1;
-//	}
-//	return 0;
-//}
-//
 /* ------------------------------------------------------------------------ */
 
-knh_bool_t knh_bytes_isNotNullType(knh_bytes_t t)
+char* knh_format_type(Ctx *ctx, char *buf, size_t bufsiz, knh_type_t type)
 {
-	if(t.len > 2 && t.buf[t.len-1] == '!' && t.buf[t.len-2] != '!') {
-		return 1;
+	if(type == TYPE_void) {
+		knh_snprintf(buf, bufsiz, "void");
+		return buf;
 	}
-	return 0;
+	knh_class_t cid = CLASS_type(type);
+	KNH_ASSERT_cid(cid);
+	char *cname = knh_tClass_CLASSN(ctx, cid);
+	if(type == NNTYPE_Int || type == NNTYPE_Float || type == NNTYPE_Boolean) {
+		knh_snprintf(buf, bufsiz, "%s", cname);
+		buf[0] = tolower(buf[0]);
+		return buf;
+	}
+	if(IS_NNTYPE(type)) {
+		knh_snprintf(buf, bufsiz, "%s!", cname);
+		return buf;
+	}
+	knh_snprintf(buf, bufsiz, "%s", cname);
+	return buf;
 }
-//
-///* ------------------------------------------------------------------------ */
-//
-//knh_int_t knh_type_compare(Ctx *ctx, knh_type_t t1, knh_type_t t2)
-//{
-////	knh_class_t cid = knh_type_tocid(t1), cid2 = knh_type_tocid(t2);
-////	knh_int_t res = knh_strcmp(CLASSN(cid), CLASSN(cid2));
-////	if(res != 0) return res;
-//	return (knh_int_t)t1 - (knh_int_t)t2;
-//}
 
 /* ======================================================================== */
 /* [movabletext] */
