@@ -157,7 +157,7 @@ knh_dict_t* knh_dict_resize(Ctx *ctx, knh_dict_t *a, size_t newsize)
 			newa = NULL;
 		}
 		for(i = newsize; i < size; i++) {
-			newh->fdict_traverse(ctx, a + i, knh_Object_RCsweep);
+			newh->fdict_traverse(ctx, a + i, knh_Object_sweep);
 		}
 		KNH_FREE(ctx, h, (sizeof(knh_dict_t) * size) + sizeof(knh_hdict_t));
 		return newa;
@@ -382,7 +382,7 @@ DictMap* new_DictMap0(Ctx *ctx, knh_int_t capacity)
 
 DictMap* new_DictMap(Ctx *ctx, knh_class_t p1, knh_int_t capacity)
 {
-	knh_class_t cid = knh_pmzclass(ctx, CLASS_DictMap, p1, CLASS_Nue);
+	knh_class_t cid = knh_class_Generics(ctx, CLASS_DictMap, p1, CLASS_Nue);
 	knh_DictMap_t *o = (knh_DictMap_t*)new_hObject(ctx, FLAG_DictMap, CLASS_DictMap, cid);
 	o->_list = knh_dict_malloc(ctx, capacity, knh_dictmap_finit, knh_dictmap_ftraverse, knh_dict_fcmp);
 	o->size = 0;
