@@ -73,6 +73,31 @@ void knh_NameSpace_setClass(Ctx *ctx, NameSpace *o, String *name, knh_class_t c)
 	knh_DictSet_set(ctx, DP(o)->name2cidDictSet, name, (knh_uint_t)(c+1));
 }
 
+/* ------------------------------------------------------------------------ */
+
+static
+knh_bytes_t knh_NameSpace_firstType(Ctx *ctx, NameSpace *o, knh_bytes_t name, knh_type_t *type)
+{
+	int i, level = 1;
+	for(i = 0; i < (int)name.len; i++) {
+		int ch = name.buf[i];
+		if(ch == '(' || ch == '<') level++;
+		if(ch == ')' || ch == '>') level--;
+		if((ch == ',' && level == 1) ||((ch == '>' || ch == ')') && level == 0)) {
+			knh_bytes_t t = {name.buf, i};
+			*type = knh_NameSpace_getType(ctx, o, t);
+			if(*type == CLASS_unknown) {
+				*type = TYPE_Any;
+			}
+			name.buf = name.buf+i+1;
+			name.len = name.len-(i+1);
+			return name;
+		}
+	}
+	return STEXT("");
+}
+
+
 /* ------------------------------------------------------------------------ */
 /* @ method Class NameSpace.getClass(String name) */
 
@@ -85,21 +110,41 @@ knh_class_t knh_NameSpace_getClass(Ctx *ctx, NameSpace *o, knh_bytes_t name)
 		return CLASS_This;
 	}
 
-	knh_index_t loc = knh_bytes_index(name, '<');
-	if(loc > 0) {
-		TODO(); // This strips C<T> to C */
-		name = knh_bytes_first(name, loc);
+	{
+		NameSpace *ns = o;
+		while(1) {
+			knh_uint_t cid = knh_DictSet_get__b(DP(ns)->name2cidDictSet, name);
+			if(cid > 0) return (knh_class_t)(cid-1);
+			if(IS_NULL(DP(ns)->parent)) {
+				goto L_GENPART;
+			}
+			ns = DP(ns)->parent;
+		}
 	}
 
-	NameSpace *ns = o;
-	L_TAIL:;
-	knh_uint_t cid = knh_DictSet_get__b(DP(ns)->name2cidDictSet, name);
-	if(cid > 0) return (knh_class_t)(cid-1);
-	if(IS_NULL(DP(ns)->parent)) {
-		return CLASS_unknown;
+	L_GENPART:
+	if(o != knh_rootNameSpace) {
+		knh_index_t loc = knh_bytes_index(name, '<');
+		if(loc > 0) {
+			knh_class_t bcid = knh_NameSpace_getClass(ctx, o, knh_bytes_first(name, loc));
+			KNH_ASSERT_cid(bcid);
+			if(!knh_Class_isGenerics(bcid)) return bcid;
+			knh_class_t p1 = CLASS_Any, p2 = ctx->tClass[bcid].p2;
+			name = knh_NameSpace_firstType(ctx, o, knh_bytes_last(name, loc+1), &p1);
+			knh_NameSpace_firstType(ctx, o, name, &p2);
+			return knh_class_Generics(ctx, bcid, CLASS_type(p1), CLASS_type(p2));
+		}
+		loc = knh_bytes_index(name, '(');
+		if(loc > 0) {
+			knh_type_t r0 = knh_NameSpace_getType(ctx, o, knh_bytes_first(name, loc));
+			knh_type_t t1 = TYPE_void, t2 = TYPE_void, t3 = TYPE_void;
+			name = knh_NameSpace_firstType(ctx, o, knh_bytes_last(name, loc+1), &t1);
+			name = knh_NameSpace_firstType(ctx, o, name, &t2);
+			knh_NameSpace_firstType(ctx, o, name, &t3);
+			return knh_class_Closure(ctx, r0, t1, t2, t3);
+		}
 	}
-	ns = DP(ns)->parent;
-	goto L_TAIL;
+	return CLASS_unknown;
 }
 
 /* ------------------------------------------------------------------------ */
