@@ -157,8 +157,8 @@ void knh_Asm_prepare(Ctx *ctx, Asm *abr, Method *mtd, Stmt *stmt)
 
 	DP(abr)->fileid  = SP(stmt)->fileid;
 	DP(abr)->line   = 0;
-	knh_Bytes_clear(DP(abr)->elf);
-	knh_Bytes_clear(DP(abr)->dwarf);
+	knh_Bytes_clear(DP(abr)->elf, 0);
+	knh_Bytes_clear(DP(abr)->dwarf, 0);
 	DP(abr)->prev_op = NULL;
 	knh_Asm_setCancelled(abr, 0);
 }
@@ -200,8 +200,8 @@ void knh_Asm_gc(Ctx *ctx, Asm *abr)
 	}
 	DP(abr)->labelmax = 0;
 	knh_Array_clear(ctx, DP(abr)->lstacks);
-	knh_Bytes_clear(DP(abr)->elf);
-	knh_Bytes_clear(DP(abr)->dwarf);
+	knh_Bytes_clear(DP(abr)->elf, 0);
+	knh_Bytes_clear(DP(abr)->dwarf, 0);
 }
 
 
@@ -216,7 +216,7 @@ void knh_Asm_finish(Ctx *ctx, Asm *abr)
 	KNH_ASM_HALT_(ctx, abr);
 
 	if(knh_Asm_isCancelled(abr)) {
-		knh_code_traverse(ctx, (knh_code_t*)knh_Bytes_tochar(DP(abr)->elf), knh_Object_RCsweep);
+		knh_code_traverse(ctx, (knh_code_t*)knh_Bytes_tochar(DP(abr)->elf), knh_Object_sweep);
 		knh_Asm_gc(ctx, abr);
 		return;
 	}
