@@ -69,6 +69,7 @@ knh_methodn_t knh_Token_getmn(Ctx *ctx, Token *o)
 		return DP(o)->mn;
 	}
 	else if(SP(o)->tt == TT_FN) {
+		DBG2_ABORT();
 		knh_methodn_t mn = FIELDN_UNMASK(DP(o)->fn);
 		if(knh_Token_isGetter(o)) {
 			return mn | KNH_FLAG_MN_GETTER;
@@ -95,13 +96,14 @@ knh_methodn_t knh_Token_getmn(Ctx *ctx, Token *o)
 		if(idx != -1) {
 			name = knh_bytes_last(name, idx+1);
 		}
-		return knh_tName_getMethodn(ctx, name, METHODN_NEWID);
-
+		knh_methodn_t mn = knh_tName_getMethodn(ctx, name, METHODN_NEWID);
+		return mn;
 	}
 	else if(SP(o)->tt == TT_MT) {
 		return knh_tName_getMethodn(ctx, name, METHODN_NEWID) | KNH_FLAG_MN_MOVTEXT;
 	}
 	DBG2_P("SP(o)->tt=%s, '%s'", knh_token_tochar(SP(o)->tt), (char*)name.buf);
+	DBG2_ABORT();
 	return knh_tName_getMethodn(ctx, name, METHODN_NEWID);
 }
 
@@ -136,10 +138,10 @@ knh_type_t knh_Token_gettype(Ctx *ctx, Token *o, NameSpace *ns, knh_class_t defc
 			}
 		}
 		if(knh_Token_isIteratorType(o)) {
-			cid = knh_pmzclass(ctx, CLASS_Iterator, cid, CLASS_Any);
+			cid = knh_class_Generics(ctx, CLASS_Iterator, cid, CLASS_Any);
 		}
 		else if(knh_Token_isArrayType(o)) {
-			cid = knh_pmzclass(ctx, CLASS_Array, cid, CLASS_Any);
+			cid = knh_class_Generics(ctx, CLASS_Array, cid, CLASS_Any);
 		}
 		if(knh_Token_isNotNullType(o)) {
 			return NNTYPE_cid(cid);
@@ -172,19 +174,19 @@ knh_class_t knh_Token_getcid(Ctx *ctx, Token *o, NameSpace *ns, knh_class_t defc
 			}
 			cid = defc;
 			if(knh_Token_isIteratorType(o)) {
-				cid = knh_pmzclass(ctx, CLASS_Iterator, cid, CLASS_Nue);
+				cid = knh_class_Generics(ctx, CLASS_Iterator, cid, CLASS_Nue);
 			}
 			else if(knh_Token_isArrayType(o)) {
-				cid = knh_pmzclass(ctx, CLASS_Array, cid, CLASS_Nue);
+				cid = knh_class_Generics(ctx, CLASS_Array, cid, CLASS_Nue);
 			}
 			knh_Token_perror(ctx, o, KMSG_UCLASSN);
 			knh_Token_perrata(ctx, o, CLASSN(cid));
 		}
 		if(knh_Token_isIteratorType(o)) {
-			cid = knh_pmzclass(ctx, CLASS_Iterator, cid, CLASS_Nue);
+			cid = knh_class_Generics(ctx, CLASS_Iterator, cid, CLASS_Nue);
 		}
 		else if(knh_Token_isArrayType(o)) {
-			cid = knh_pmzclass(ctx, CLASS_Array, cid, CLASS_Nue);
+			cid = knh_class_Generics(ctx, CLASS_Array, cid, CLASS_Nue);
 		}
 		return cid;
 	}
@@ -585,6 +587,18 @@ int knh_TokenCONSTN_typing(Ctx *ctx, Token *o, Asm *abr, NameSpace *ns)
 /* ------------------------------------------------------------------------ */
 /* [CLASSID] */
 
+static
+void knh_Token_toCLASSID(Ctx *ctx, Token *o, knh_class_t cid)
+{
+	KNH_ASSERT_cid(cid);
+	SP(o)->tt = TT_CLASSID;
+	DP(o)->cid = cid;
+	DP(o)->type = NNTYPE_Class;
+	KNH_SETv(ctx, DP(o)->data, ctx->tClass[cid].class_cid);
+}
+
+/* ------------------------------------------------------------------------ */
+
 static
 int knh_TokenTYEPN_typing(Ctx *ctx, Token *o, Asm *abr, NameSpace *ns)
 {
@@ -601,11 +615,7 @@ int knh_TokenTYEPN_typing(Ctx *ctx, Token *o, Asm *abr, NameSpace *ns)
 		knh_Token_perror(ctx, o, KMSG_UCLASSN);
 		return 0;
 	}
-	KNH_ASSERT_cid(cid);
-	SP(o)->tt = TT_CLASSID;
-	DP(o)->cid = cid;
-	DP(o)->type = NNTYPE_Class;
-	KNH_SETv(ctx, DP(o)->data, ctx->tClass[cid].class_cid);
+	knh_Token_toCLASSID(ctx, o, cid);
 	return 1;
 }
 
@@ -635,7 +645,6 @@ int knh_TokenCMETHODN_typing(Ctx *ctx, Token *o, NameSpace *ns)
 	return 1;
 }
 
-/* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 /* [NUM] */
 
@@ -816,7 +825,7 @@ int knh_Token_typing(Ctx *ctx, Token *tk, Asm *abr, NameSpace *ns, knh_type_t re
 	case TT_FN :
 	case TT_NAME: {
 		int res = knh_TokenNAME_typing(ctx, tk, abr);
-		//DBG2_P("NAME tt=%s, index=%d", knh_token_tochar(SP(tk)->tt), DP(tk)->index)
+		//DBG2_P("NAME tt=%s, index=%d, res=%d", knh_token_tochar(SP(tk)->tt), DP(tk)->index, res);
 		return res;
 	}
 	case TT_NUM:
@@ -955,7 +964,6 @@ knh_type_t TERMs_gettype(Stmt *stmt, size_t n)
 
 /* ------------------------------------------------------------------------ */
 
-static
 char * knh_format_mtdparam(Ctx *ctx, char *buf, size_t bufsiz, Method *mtd, int n)
 {
 	char bufa[20];
@@ -1408,97 +1416,6 @@ knh_Class_t *new_Class__type(Ctx *ctx, knh_type_t type)
 /* ------------------------------------------------------------------------ */
 /* [CALL] */
 
-static
-Term *knh_StmtCALLBASE_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_methodn_t mn)
-{
-	Token *tk = DP(stmt)->tokens[1];
-	if(IS_Stmt(tk) || SP(tk)->tt != TT_ASIS) {
-		if(!TERMs_typing(ctx, stmt, 1, abr, ns, CLASS_Any, TWARN_)) {
-			return NULL;
-		}
-		if(TERMs_isCLASSID(stmt, 1)) {
-			Token *tk = DP(stmt)->tokens[1];
-			knh_Token_toDEFVAL(tk, DP(tk)->cid);
-		}
-		return TM(stmt);
-	}
-
-	/* (func _ ...) */
-	/* built-in function */
-	int pe = KMSG_EBUILTINFUNC;
-	if(mn == METHODN_typeof) {
-		if(DP(stmt)->size != 3) goto L_ERROR;
-		if(!TERMs_typing(ctx, stmt, 2, abr, ns, CLASS_Any, TWARN_)) {
-			knh_Token_setCONST(ctx, tk, UP(new_Class__type(ctx, TERMs_gettype(stmt, 2))));
-			return TM(tk);
-		}
-		return NULL;
-	}
-	else if(mn == METHODN_default) {
-		if(DP(stmt)->size != 3) goto L_ERROR;
-		if(!TERMs_typing(ctx, stmt, 2, abr, ns, CLASS_Any, TWARN_)) {
-			if(TERMs_isCLASSID(stmt, 2)) {
-				tk = DP(stmt)->tokens[2];
-				knh_Token_toDEFVAL(tk, DP(tk)->cid);
-			}
-			else {
-				knh_Token_toDEFVAL(tk, TERMs_getcid(stmt, 2));
-			}
-			return TM(tk);
-		}
-		return NULL;
-	}
-	else if(mn == METHODN_defined) {
-		if(DP(stmt)->size != 3) goto L_ERROR;
-		if(TERMs_typing(ctx, stmt, 2, abr, ns, CLASS_Any, TWARN_)) {
-			knh_Token_setCONST(ctx, tk, KNH_TRUE);
-		}
-		else {
-			knh_Token_setCONST(ctx, tk, KNH_FALSE);
-		}
-		return TM(tk);
-	}
-	else {
-		char bufmn[CLASSNAME_BUFSIZ];
-		knh_format_methodn(ctx, bufmn, sizeof(bufmn), mn);
-
-		//DEBUG3("1. lookup function.. %s()", bufmn);
-		knh_class_t mtd_cid = knh_NameSpace_getFuncClass(ctx, ns, B(bufmn));
-		if(mtd_cid != CLASS_unknown) {
-			KNH_ASSERT_cid(mtd_cid);
-			knh_Token_toDEFVAL(tk, mtd_cid);
-			return TM(stmt);
-		}
-
-		//DEBUG3("2. lookup this.%s()", bufmn);
-		if(DP(abr)->vars[0].fn == FIELDN_this) {
-			mtd_cid = CLASS_type(DP(abr)->vars[0].type);
-			Method *mtd = knh_Class_getMethod(ctx, mtd_cid, mn);
-			if(IS_NOTNULL(mtd)) {
-				knh_Token_toSTACK(ctx, tk, 0, DP(abr)->vars[0].type);
-				return TM(stmt);
-			}
-		}
-
-		//DEBUG3("3. lookup script function %s()", bufmn);
-		Script *scr = knh_Asm_getScript(ctx, abr);
-		mtd_cid = knh_Object_cid(scr);
-		Method *mtd = knh_Class_getMethod(ctx, mtd_cid, mn);
-		//KNH_ASSERT(IS_NOTNULL(mtd));
-		if(IS_NOTNULL(mtd)) {
-			knh_Token_setCONST(ctx, tk, UP(scr));
-			return TM(stmt);
-		}
-		pe = KMSG_UMETHODN;
-	}
-
-	L_ERROR:;
-	knh_Asm_perror(ctx, abr, pe, sToken(DP(stmt)->tokens[0]));
-	return NULL;
-}
-
-/* ------------------------------------------------------------------------ */
-
 static
 void KNH_BOX(Ctx *ctx, knh_sfp_t *sfp, knh_type_t type)
 {
@@ -1550,6 +1467,17 @@ static Term *knh_StmtCALL_toCONST(Ctx *ctx, Stmt *stmt, Method *mtd)
 
 /* ------------------------------------------------------------------------ */
 
+static
+void knh_Token_toMTD(Ctx *ctx, Token *tk, knh_methodn_t mn, Method *mtd)
+{
+	SP(tk)->tt = TT_MN;
+	DP(tk)->mn = mn;
+	KNH_SETv(ctx, DP(tk)->data, mtd);
+	DP(tk)->type = TYPE_Method;
+}
+
+/* ------------------------------------------------------------------------ */
+
 static
 Term *knh_StmtPARAMS_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_class_t mtd_cid, Method *mtd)
 {
@@ -1596,13 +1524,293 @@ Term *knh_StmtPARAMS_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_c
 
 /* ------------------------------------------------------------------------ */
 
-static
-void knh_Token_toMTD(Ctx *ctx, Token *tk, knh_methodn_t mn, Method *mtd)
+static int knh_type_isClosure(Ctx *ctx, knh_type_t type)
 {
-	SP(tk)->tt = TT_MN;
-	DP(tk)->mn = mn;
-	KNH_SETv(ctx, DP(tk)->data, mtd);
-	DP(tk)->type = TYPE_Method;
+	knh_class_t cid = CLASS_type(type);
+	KNH_ASSERT_cid(cid);
+	return (ctx->tClass[cid].bcid == CLASS_Closure);
+}
+
+/* ------------------------------------------------------------------------ */
+
+static int knh_TokenNAME_isClosure(Ctx *ctx, Token *tk, Asm *abr)
+{
+	knh_fieldn_t fnq = knh_Token_getfnq(ctx, tk);
+	if(fnq == FIELDN_NONAME) return 0;
+
+	if(FIELDN_IS_U1(fnq) || FIELDN_IS_SUPER(fnq)) goto L_FIELD;  /* _name */
+	if(FIELDN_IS_U2(fnq)) goto L_GLOBAL; /* __name */
+
+	knh_index_t idx = knh_Asm_indexOfVariable(abr, FIELDN_UNMASK(fnq));
+	if(idx != -1) {
+		knh_cfield_t *cf = knh_Asm_cfieldOfVariable(abr, idx);
+		knh_type_t type = knh_pmztype_totype(ctx, cf->type, DP(abr)->this_cid);
+		if(knh_type_isClosure(ctx, type)) {
+			knh_Token_toSTACK(ctx, tk, idx, type);
+			return 1;
+		}
+		return 0;
+	}
+
+	L_FIELD:;
+	idx = knh_Class_queryField(ctx, DP(abr)->this_cid, fnq);
+	if(idx != -1) {
+		knh_cfield_t *cf = knh_Class_fieldAt(ctx, DP(abr)->this_cid, idx);
+		knh_type_t type = knh_pmztype_totype(ctx, cf->type, DP(abr)->this_cid);
+		if(knh_type_isClosure(ctx, type)) {
+			knh_Token_toFIELD(ctx, tk, idx, type);
+			return 1;
+		}
+		return 0;
+	}
+
+	L_GLOBAL:;
+	{
+		Script *scr = knh_Asm_getScript(ctx, abr);
+		idx = knh_Class_queryField(ctx, knh_Object_cid(scr), fnq);
+		if(idx != -1) {
+			knh_cfield_t *cf = knh_Class_fieldAt(ctx, knh_Object_cid(scr), idx);
+			knh_type_t type = knh_pmztype_totype(ctx, cf->type, knh_Object_cid(scr));
+			if(knh_type_isClosure(ctx, type)) {
+				knh_Token_toGLOBAL(ctx, tk, idx, type);
+				knh_Asm_globalIndex(ctx, abr);
+				return 1;
+			}
+		}
+	}
+	return 0;
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+Term *knh_StmtINVOKE_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
+{
+	knh_class_t cid = TERMs_getcid(stmt, 0);
+	KNH_ASSERT_cid(cid); DBG2_ASSERT(knh_type_isClosure(ctx, cid));
+	/* 0 1 2 3 4 .. 5 */
+	if(cid == CLASS_Closure /* || DP(stmt)->size > 4 */) {
+		size_t i;
+		for(i = 0; i < DP(stmt)->size; i++) {
+			if(!TERMs_typing(ctx, stmt, i, abr, ns, TYPE_Any, TWARN_)) {
+				return NULL;
+			}
+		}
+		knh_Stmt_setType(ctx, stmt, TYPE_Any);
+		return TM(stmt);
+	}
+	knh_Stmt_setType(ctx, stmt, ctx->tClass[cid].r0);
+
+	if(ctx->tClass[cid].p1 != TYPE_void) {
+		if(!(DP(stmt)->size > 2)) {
+			knh_Stmt_add(ctx, stmt, UP(new_TokenNULL(ctx, FL(stmt), ctx->tClass[cid].p1)));
+		}
+		if(!TERMs_typing(ctx, stmt, 2, abr, ns, ctx->tClass[cid].p1, TCHECK_)) {
+			return NULL;
+		}
+	}
+	else {
+		DP(stmt)->size = 2;
+		return TM(stmt);
+	}
+
+	if(ctx->tClass[cid].p2 != TYPE_void) {
+		if(!(DP(stmt)->size > 3)) {
+			knh_Stmt_add(ctx, stmt, UP(new_TokenNULL(ctx, FL(stmt), ctx->tClass[cid].p2)));
+		}
+		if(!TERMs_typing(ctx, stmt, 3, abr, ns, ctx->tClass[cid].p2, TCHECK_)) {
+			return NULL;
+		}
+	}
+	else {
+		DP(stmt)->size = 3;
+		return TM(stmt);
+	}
+
+	if(ctx->tClass[cid].p3 != TYPE_void) {
+		if(!(DP(stmt)->size > 4)) {
+			knh_Stmt_add(ctx, stmt, UP(new_TokenNULL(ctx, FL(stmt), ctx->tClass[cid].p3)));
+		}
+		if(!TERMs_typing(ctx, stmt, 4, abr, ns, ctx->tClass[cid].p3, TCHECK_)) {
+			return NULL;
+		}
+		DP(stmt)->size = 5;
+		return TM(stmt);
+	}
+	else {
+		DP(stmt)->size = 4;
+		return TM(stmt);
+	}
+
+}
+
+/* ------------------------------------------------------------------------- */
+
+static
+Term *knh_StmtCALLBASE_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_methodn_t mn)
+{
+	Token *tk = DP(stmt)->tokens[1];
+	if(IS_Stmt(tk) || SP(tk)->tt != TT_ASIS) {
+		if(!TERMs_typing(ctx, stmt, 1, abr, ns, CLASS_Any, TWARN_)) {
+			return NULL;
+		}
+		if(TERMs_isCLASSID(stmt, 1)) {
+			Token *tk = DP(stmt)->tokens[1];
+			knh_Token_toDEFVAL(tk, DP(tk)->cid);
+		}
+		return TM(stmt);
+	}
+
+	/* (func _ ...) */
+	/* built-in function */
+	int pe = KMSG_EBUILTINFUNC;
+	if(mn == METHODN_typeof) {
+		if(DP(stmt)->size != 3) goto L_ERROR;
+		if(TERMs_typing(ctx, stmt, 2, abr, ns, TYPE_Any, TWARN_)) {
+			knh_Token_setCONST(ctx, tk, UP(new_Class__type(ctx, TERMs_gettype(stmt, 2))));
+			return TM(tk);
+		}
+		return NULL;
+	}
+	else if(mn == METHODN_default) {
+		if(DP(stmt)->size != 3) goto L_ERROR;
+		if(TERMs_typing(ctx, stmt, 2, abr, ns, CLASS_Any, TWARN_)) {
+			if(TERMs_isCLASSID(stmt, 2)) {
+				tk = DP(stmt)->tokens[2];
+				knh_Token_toDEFVAL(tk, DP(tk)->cid);
+			}
+			else {
+				knh_Token_toDEFVAL(tk, TERMs_getcid(stmt, 2));
+			}
+			return TM(tk);
+		}
+		return NULL;
+	}
+	else if(mn == METHODN_defined) {
+		if(DP(stmt)->size != 3) goto L_ERROR;
+		if(TERMs_typing(ctx, stmt, 2, abr, ns, CLASS_Any, TWARN_)) {
+			knh_Token_setCONST(ctx, tk, KNH_TRUE);
+		}
+		else {
+			knh_Token_setCONST(ctx, tk, KNH_FALSE);
+		}
+		return TM(tk);
+	}
+	else if(mn == METHODN_super || mn == METHODN_this) {
+		knh_class_t mtd_cid = CLASS_type(DP(abr)->vars[0].type);
+		if(mn == METHODN_super) {
+			KNH_ASSERT_cid(mtd_cid); mtd_cid = ctx->tClass[mtd_cid].supcid;
+		}
+		Token *mtk = DP(stmt)->tokens[0]; /* change */
+		SP(mtk)->tt = TT_MN;
+		DP(mtk)->mn = METHODN_new;
+		knh_Token_toSTACK(ctx, tk, 0, DP(abr)->vars[0].type);
+		return TM(stmt);
+	}
+	else if(mn == METHODN_delegate) { /* delegate */
+		knh_class_t cid;
+		if(DP(stmt)->size == 4) { /* delegate(a,f) */
+			Token *ftk = DP(stmt)->tokens[3];
+			if(!IS_Token(ftk)) goto L_ERROR;
+			knh_methodn_t mn = knh_Token_getmn(ctx, ftk);
+			if(!TERMs_typing(ctx, stmt, 2, abr, ns, TYPE_Any, TWARN_)) {
+				return NULL;
+			}
+			if(TERMs_isCLASSID(stmt, 2)) {
+				Token *ctk = DP(stmt)->tokens[2];
+				knh_Token_toDEFVAL(ctk, DP(ctk)->cid);
+			}
+			cid = TERMs_getcid(stmt, 2);
+			Method *mtd = knh_Class_getMethod(ctx, cid, mn);
+			if(IS_NULL(mtd)) { pe = KMSG_UMETHODN; goto L_ERROR; }
+			knh_Token_setCONST(ctx, DP(stmt)->tokens[3], UP(mtd));
+			cid = knh_class_MethodClosure(ctx, cid, mtd);
+		}
+		else if(DP(stmt)->size == 3) { /* delegate(f) */
+			Token *ftk = DP(stmt)->tokens[2];
+			if(!IS_Token(ftk)) goto L_ERROR;
+			knh_methodn_t mn = knh_Token_getmn(ctx, ftk);
+			cid = CLASS_type(DP(abr)->vars[0].type);
+			Method *mtd = knh_Class_getMethod(ctx, cid, mn);
+			if(IS_NULL(mtd)) {
+				Script *scr = knh_Asm_getScript(ctx, abr);
+				cid = knh_Object_cid(scr);
+				mtd = knh_Class_getMethod(ctx, cid, mn);
+				if(IS_NULL(mtd)) { pe = KMSG_UMETHODN; goto L_ERROR; }
+				knh_Token_setCONST(ctx, DP(stmt)->tokens[2], UP(scr));
+			}
+			else {
+				knh_Token_toSTACK(ctx, ftk, 0, DP(abr)->vars[0].type);
+			}
+			knh_Stmt_add(ctx, stmt, UP(new_TokenCONST(ctx, FL(stmt), UP(mtd))));
+			cid = knh_class_MethodClosure(ctx, cid, mtd);
+		}
+		else {
+			goto L_ERROR;
+		}
+		knh_Token_toCLASSID(ctx, tk, cid);
+		SP(stmt)->stt = STT_NEW;
+		Method *mtd = knh_Class_getMethod(ctx, CLASS_Closure, METHODN_new);
+		KNH_ASSERT((IS_Method(mtd)));
+		knh_Token_toMTD(ctx, DP(stmt)->tokens[0], mn, mtd);
+		//DBG2_P("cid=%s", CLASSN(cid));
+		knh_Stmt_setType(ctx, stmt, NNTYPE_cid(cid));
+		return TM(stmt);
+	}
+	else if(mn == METHODN_proceed) { /* proceed() */
+		knh_Asm_setPROCEED(abr, 1);
+		SP(stmt)->stt = STT_PROCEED;
+		Method *mtd = DP(abr)->mtd;
+		knh_Token_toMTD(ctx, DP(stmt)->tokens[0], DP(mtd)->mn, mtd);
+		knh_Token_toSTACK(ctx, tk, 0, DP(abr)->vars[0].type);
+		if(DP(stmt)->size > 1) {
+			knh_StmtPARAMS_typing(ctx, stmt, abr, ns, CLASS_type(DP(abr)->vars[0].type), mtd);
+		}
+		knh_Stmt_setType(ctx, stmt, DP(abr)->rtype);
+		return TM(stmt);
+	}
+	else {
+		char bufmn[CLASSNAME_BUFSIZ];
+		knh_format_methodn(ctx, bufmn, sizeof(bufmn), mn);
+		//DEBUG3("0. lookup closure..");
+		if(knh_TokenNAME_isClosure(ctx, DP(stmt)->tokens[0], abr)) {
+			SP(stmt)->stt = STT_INVOKE;
+			return knh_StmtINVOKE_typing(ctx, stmt, abr, ns);
+		}
+
+		//DEBUG3("1. lookup function.. %s()", bufmn);
+		knh_class_t mtd_cid = knh_NameSpace_getFuncClass(ctx, ns, B(bufmn));
+		if(mtd_cid != CLASS_unknown) {
+			KNH_ASSERT_cid(mtd_cid);
+			knh_Token_toDEFVAL(tk, mtd_cid);
+			return TM(stmt);
+		}
+
+		//DEBUG3("2. lookup this.%s()", bufmn);
+		if(DP(abr)->vars[0].fn == FIELDN_this) {
+			mtd_cid = CLASS_type(DP(abr)->vars[0].type);
+			Method *mtd = knh_Class_getMethod(ctx, mtd_cid, mn);
+			if(IS_NOTNULL(mtd)) {
+				knh_Token_toSTACK(ctx, tk, 0, DP(abr)->vars[0].type);
+				return TM(stmt);
+			}
+		}
+
+		//DEBUG3("3. lookup script function %s()", bufmn);
+		Script *scr = knh_Asm_getScript(ctx, abr);
+		mtd_cid = knh_Object_cid(scr);
+		Method *mtd = knh_Class_getMethod(ctx, mtd_cid, mn);
+		//KNH_ASSERT(IS_NOTNULL(mtd));
+		if(IS_NOTNULL(mtd)) {
+			knh_Token_setCONST(ctx, tk, UP(scr));
+			return TM(stmt);
+		}
+		pe = KMSG_UMETHODN;
+	}
+
+	L_ERROR:;
+	knh_Asm_perror(ctx, abr, pe, sToken(DP(stmt)->tokens[0]));
+	return NULL;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1613,10 +1821,13 @@ Term *knh_StmtCALL_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_cla
 	DEBUG3_ASSERT(DP(stmt)->size > 1);
 	knh_methodn_t mn = knh_Token_getmn(ctx, DP(stmt)->tokens[0]);
 	Term *tm = knh_StmtCALLBASE_typing(ctx, stmt, abr, ns, mn);
-	if(tm == NULL || IS_Token(tm)) {
+	if(tm == NULL || IS_Token(tm) || knh_Stmt_isTyped(stmt)) {
 		return tm;
 	}
 
+	if(mn == METHODN_super || mn == METHODN_this) {
+		mn = METHODN_new;
+	}
 	knh_type_t    btype   = TERMs_gettype(stmt, 1);
 	knh_class_t   mtd_cid = CLASS_type(btype);
 	Method *mtd = knh_Class_getMethod(ctx, mtd_cid, mn);
@@ -1628,7 +1839,13 @@ Term *knh_StmtCALL_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns, knh_cla
 	knh_Token_toMTD(ctx, DP(stmt)->tokens[0], mn, mtd);
 
 	if(IS_NOTNULL(mtd)) {
-		return knh_StmtPARAMS_typing(ctx, stmt, abr, ns, mtd_cid, mtd);
+		Term *tm = knh_StmtPARAMS_typing(ctx, stmt, abr, ns, mtd_cid, mtd);
+		if(tm != NULL && IS_Stmt(tm)) {
+			if(knh_methodn_isNew(ctx, mn)) {
+				knh_Stmt_setType(ctx, stmt, TYPE_void);
+			}
+		}
+		return TM(tm);
 	}
 	else {
 		int i;
@@ -2921,10 +3138,10 @@ Term * knh_StmtMETHOD_typing(Ctx *ctx, Stmt *stmt, Asm *abr, NameSpace *ns)
 	}
 
 	L_PERROR:;
-	knh_wbuf_t wb = knh_Context_wbuf(ctx);
-	knh_Method__k(ctx, mtd, wb.w, (String*)KNH_NULL);
-	knh_Asm_perror(ctx, abr, pe, knh_wbuf_top(wb));
-	knh_wbuf_clear(wb);
+	knh_cwb_t cwb = new_cwb(ctx);
+	knh_Method__k(ctx, mtd, cwb.w, (String*)KNH_NULL);
+	knh_Asm_perror(ctx, abr, pe, knh_cwb_tochar(cwb));
+	knh_cwb_clear(cwb);
 	return NULL;
 }
 
