@@ -138,15 +138,15 @@ METHOD code_PINIO(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 
-METHOD code_SETEBP(Ctx *ctx, knh_sfp_t *sfp)
+METHOD code_SETESP(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KLR_SETEBP(ctx, 100);
+	KLR_SETESP(ctx, 100);
 }
 
 
-METHOD code_CHECKEBP(Ctx *ctx, knh_sfp_t *sfp)
+METHOD code_CHECKESP(Ctx *ctx, knh_sfp_t *sfp)
 {
-	KLR_CHECKEBP(ctx, 100);
+	KLR_CHECKESP(ctx, 100);
 }
 
 
