@@ -1062,6 +1062,8 @@ static char* knh_stmt_array_tochar[] = {
     "pcall",  /* pcall */ 
     "mt",  /* mt */ 
     "op",  /* op */ 
+    "invoke",  /* invoke */ 
+    "proceed",  /* proceed */ 
     "tri",  /* tri */ 
     "alt",  /* alt */ 
     "and",  /* and */ 
@@ -1132,6 +1134,8 @@ static size_t knh_stmt_array_size[] = {
     7,  /* pcall */ 
     3,  /* mt */ 
     7,  /* op */ 
+    1,  /* invoke */ 
+    2,  /* proceed */ 
     3,  /* tri */ 
     5,  /* alt */ 
     5,  /* and */ 
@@ -1203,6 +1207,8 @@ static int knh_stmt_array_isExpr[] = {
     1,  /* pcall */ 
     1,  /* mt */ 
     1,  /* op */ 
+    1,  /* invoke */ 
+    1,  /* proceed */ 
     1,  /* tri */ 
     1,  /* alt */ 
     1,  /* and */ 
