@@ -43,8 +43,23 @@ extern "C" {
 /* ======================================================================== */
 /* [option] */
 
-static int optionAdmin = 0;
-static int optionCompiling = 0;
+static int debugMode2 = 0;
+
+/* ----------------------------------------------------------------------- */
+
+static
+void konoha_checksize()
+{
+	fprintf(stderr, "sizeof(knh_int_t)=%lu, sizeof(void*)=%lu\n", sizeof(knh_int_t), sizeof(void*));
+	KNH_ASSERT(sizeof(knh_int_t) == sizeof(void*));
+	fprintf(stderr, "sizeof(knh_integer_t)=%lu, sizeof(knh_float_t)=%lu\n", sizeof(knh_integer_t), sizeof(knh_float_t));
+	KNH_ASSERT(sizeof(knh_integer_t) <= sizeof(knh_float_t));
+	fprintf(stderr, "sizeof(knh_sfp_t)=%lu, sizeof(Ctx)=%lu\n", sizeof(knh_sfp_t), sizeof(knh_Context_t));
+
+	fprintf(stderr, "sizeof(Object)=%lu FASTMALLOC=%lu\n", sizeof(knh_Object_t), KNH_FASTMALLOC_SIZE);
+	fprintf(stderr, "sizeof(Int)=%lu\n", sizeof(knh_Int_t));
+	KNH_ASSERT(sizeof(knh_Int_t) <= sizeof(knh_Object_t) - SIZEOF_FOBJECT_T);
+}
 
 /* ----------------------------------------------------------------------- */
 
@@ -56,13 +71,15 @@ KNHAPI(int) konoha_parseopt(konoha_t konoha, int argc, char **argv)
 		char *t = argv[n];
 		if(t[0] != '-') return n;
 		if(t[1] == 's' && t[2] == 0) {
-			optionAdmin = 1; continue;
 			knh_Context_setAdmin(konoha.ctx, 1);
 		}
 		if(t[1] == 'c' && t[2] == 0) {
-			optionCompiling = 1; continue;
 			knh_Context_setCompiling(konoha.ctx, 1);
 		}
+		if(t[1] == 'd' && t[2] == '2' && t[3] == 0) {
+			debugMode2 = 1;
+			konoha_checksize();
+		}
 	}
 	return n;
 }
@@ -75,11 +92,11 @@ KNHAPI(char*) konoha_eval(konoha_t konoha, char *script)
 	KONOHA_CHECK(konoha, NULL);
 	Ctx *ctx = konoha.ctx;
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-	knh_wbuf_t wbuf = knh_Context_wbuf(ctx);
-	knh_Bytes_write(ctx, wbuf.ba, B(script));
-	knh_Bytes_putc(ctx, wbuf.ba, ';');
-	knh_Bytes_putc(ctx, wbuf.ba, '\n');
-	InputStream *in = new_BytesInputStream(ctx, wbuf.ba, wbuf.pos, knh_Bytes_size(wbuf.ba));
+	knh_cwb_t cwb = new_cwb(ctx);
+	knh_Bytes_write(ctx, cwb.ba, B(script));
+	knh_Bytes_putc(ctx, cwb.ba, ';');
+	knh_Bytes_putc(ctx, cwb.ba, '\n');
+	InputStream *in = new_BytesInputStream(ctx, cwb.ba, cwb.pos, knh_Bytes_size(cwb.ba));
 	KNH_LPUSH(ctx, in);
 	DP(in)->fileid = knh_tName_getFileId(ctx, STEXT("(eval)"));
 	DP(in)->line = 0;
@@ -91,7 +108,7 @@ KNHAPI(char*) konoha_eval(konoha_t konoha, char *script)
 	KNH_SETv(ctx, ((Context*)ctx)->err, out);
 
 	konohac_eval(ctx, TS_main, in);
-	knh_Bytes_subclear(wbuf.ba, wbuf.pos);
+	knh_cwb_clear(cwb);
 	knh_OutputStream_flush(ctx, out);
 	KNH_LOCALBACK(ctx, lsfp);
 	return (char*)ba->buf;
@@ -255,10 +272,10 @@ KNHAPI(void) konoha_shell(konoha_t konoha)
 	Ctx *ctx = konoha.ctx;
 	knh_System__dump(ctx, NULL, KNH_STDOUT, (String*)KNH_NULL);
 	int linenum, linecnt = 0;
-	knh_wbuf_t cb = knh_Context_wbuf(ctx);
+	knh_cwb_t cwb = new_cwb(ctx);
 	char *ln = NULL;
 	knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-	KNH_LPUSH(ctx, KNH_NULL);  // step ebp++;
+	KNH_LPUSH(ctx, KNH_NULL);  // step esp++;
 
 	START_LINE:;
 	if(ln != NULL) free(ln);
@@ -277,25 +294,25 @@ KNHAPI(void) konoha_shell(konoha_t konoha)
 			knh_Context_clearstack(ctx);
 			return;
 		}
-		knh_Bytes_write(ctx, cb.ba, B(ln));
-		nest = konoha_shell_checkline(knh_wbuf_tobytes(cb));
+		knh_Bytes_write(ctx, cwb.ba, B(ln));
+		nest = konoha_shell_checkline(knh_cwb_tobytes(cwb));
 		if(nest == 0) break;
 		if(nest < 0) {
 			knh_println(ctx, KNH_STDOUT, STEXT("(Canceled)"));
-			knh_Bytes_subclear(cb.ba, cb.pos);
+			knh_cwb_clear(cwb);
 			goto START_LINE;
 		}else {
-			knh_Bytes_putc(ctx, cb.ba, '\n');
+			knh_Bytes_putc(ctx, cwb.ba, '\n');
 			if(ln != NULL) free(ln);
 			ln = knh_readline("... ");
 			linecnt++;
 		}
 	}
-	if(cb.pos < knh_Bytes_size(cb.ba)) {
-		knh_bytes_t t = knh_wbuf_tobytes(cb);
+	if(cwb.pos < knh_Bytes_size(cwb.ba)) {
+		knh_bytes_t t = knh_cwb_tobytes(cwb);
 		if(ISB(t, "exit") || ISB(t, "quit") || ISB(t, "bye")) {
 			if(ln != NULL) free(ln);
-			knh_Bytes_subclear(cb.ba, cb.pos);
+			knh_cwb_clear(cwb);
 			KNH_LOCALBACK(ctx, lsfp);
 			knh_Context_clearstack(ctx);
 			return;
@@ -307,8 +324,8 @@ KNHAPI(void) konoha_shell(konoha_t konoha)
 				if(*p == ';') *p = ' ';
 				p++;
 			}
-			knh_Bytes_subclear(cb.ba, cb.pos);
-			knh_Bytes_write(ctx, cb.ba, B(bufc));
+			knh_cwb_clear(cwb);
+			knh_Bytes_write(ctx, cwb.ba, B(bufc));
 		}
 		else if(knh_bytes_startsWith(t, STEXT("dump "))) {
 			char bufc[CLASSNAME_BUFSIZ], *p = bufc;
@@ -317,19 +334,19 @@ KNHAPI(void) konoha_shell(konoha_t konoha)
 				if(*p == ';') *p = ' ';
 				p++;
 			}
-			knh_Bytes_subclear(cb.ba, cb.pos);
-			knh_Bytes_write(ctx, cb.ba, B(bufc));
+			knh_cwb_clear(cwb);
+			knh_Bytes_write(ctx, cwb.ba, B(bufc));
 		}
-		knh_Bytes_putc(ctx, cb.ba, ';');
-		knh_Bytes_putc(ctx, cb.ba, '\n');
+		knh_Bytes_putc(ctx, cwb.ba, ';');
+		knh_Bytes_putc(ctx, cwb.ba, '\n');
 		{
-			InputStream *in = new_BytesInputStream(ctx, cb.ba, cb.pos, knh_Bytes_size(cb.ba));
+			InputStream *in = new_BytesInputStream(ctx, cwb.ba, cwb.pos, knh_Bytes_size(cwb.ba));
 			KNH_MOV(ctx, lsfp[0].o, in);
 			DP(in)->fileid = knh_tName_getFileId(ctx, STEXT("(shell)"));
 			DP(in)->line = linenum;
 			knh_InputStream_setEncoding(ctx, in, knh_systemEncoding);
 			konohac_eval(ctx, TS_main, in);
-			knh_Bytes_subclear(cb.ba, cb.pos);
+			knh_cwb_clear(cwb);
 		}
 	}
 	goto START_LINE;
@@ -342,11 +359,11 @@ METHOD knh__Script_eval(Ctx *ctx, knh_sfp_t *sfp)
 {
 	if(IS_NOTNULL(sfp[1].s)) {
 		knh_sfp_t *lsfp = KNH_LOCAL(ctx);
-		knh_wbuf_t wbuf = knh_Context_wbuf(ctx);
-		knh_Bytes_write(ctx, wbuf.ba, knh_String_tobytes(sfp[1].s));
-		knh_Bytes_putc(ctx, wbuf.ba, ';');
-		knh_Bytes_putc(ctx, wbuf.ba, '\n');
-		InputStream *in = new_BytesInputStream(ctx, wbuf.ba, wbuf.pos, knh_Bytes_size(wbuf.ba));
+		knh_cwb_t cwb = new_cwb(ctx);
+		knh_Bytes_write(ctx, cwb.ba, knh_String_tobytes(sfp[1].s));
+		knh_Bytes_putc(ctx, cwb.ba, ';');
+		knh_Bytes_putc(ctx, cwb.ba, '\n');
+		InputStream *in = new_BytesInputStream(ctx, cwb.ba, cwb.pos, knh_Bytes_size(cwb.ba));
 		KNH_LPUSH(ctx, in);
 		if(IS_NULL(sfp[2].o)) {
 			DP(in)->fileid = knh_tName_getFileId(ctx, STEXT("(eval)"));
@@ -361,7 +378,7 @@ METHOD knh__Script_eval(Ctx *ctx, knh_sfp_t *sfp)
 			DP(in)->fileid = (knh_fileid_t)p_int(sfp[3]);
 		}
 		konohac_eval(ctx, TS_main, in);
-		knh_Bytes_subclear(wbuf.ba, wbuf.pos);
+		knh_cwb_clear(cwb);
 		KNH_LOCALBACK(ctx, lsfp);
 	}
 	KNH_RETURN_void(ctx, sfp);
