@@ -558,8 +558,12 @@ void knh_Tuple2_traverse(Ctx *ctx, knh_Tuple2_t *t, knh_ftraverse ftr)
 static
 void knh_Array_init(Ctx *ctx, knh_Array_t *a, int init)
 {
-	a->list = knh_oarray_malloc(ctx, (size_t)init, KNH_NULL);
+	a->list = NULL;
+	a->capacity = 0;
 	a->size = 0;
+	if(init > 0) {
+		knh_Array_grow(ctx, a, init, KNH_NULL);
+	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -567,7 +571,100 @@ void knh_Array_init(Ctx *ctx, knh_Array_t *a, int init)
 static
 void knh_Array_traverse(Ctx *ctx, knh_Array_t *a, knh_ftraverse ftr)
 {
-	knh_oarray_traverse(ctx, a->list, ftr);
+	size_t i;
+	for(i = 0; i < a->capacity; i++) {
+		ftr(ctx, a->list[i]);
+	}
+	if(a->capacity > 0 && IS_SWEEP(ftr)) {
+		if(knh_Array_isDim(a)) {
+			knh_array_dfree(ctx, a->list, a->capacity, sizeof(Object*));
+		}
+		else {
+			KNH_FREE(ctx, a->list, a->capacity * sizeof(Object*));
+		}
+		a->size = 0;
+		a->capacity = 0;
+	}
+}
+
+/* ======================================================================== */
+/* IArray */
+
+#define knh_IArray_init_ NULL
+#define knh_IArray_copy NULL
+#define knh_IArray_traverse_ NULL
+#define knh_IArray_compareTo NULL
+#define knh_IArray_hashCode NULL
+#define knh_IArray_newClass NULL
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_IArray_init(Ctx *ctx, knh_IArray_t *a, int init)
+{
+	a->ilist = NULL;
+	a->capacity = 0;
+	a->size = 0;
+	if(init > 0) {
+		knh_IArray_grow(ctx, a, init, 0);
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_IArray_traverse(Ctx *ctx, knh_IArray_t *a, knh_ftraverse ftr)
+{
+	if(a->capacity > 0 && IS_SWEEP(ftr)) {
+		if(knh_IArray_isDim(a)) {
+			knh_array_dfree(ctx, a->ilist, a->capacity, sizeof(knh_integer_t));
+		}
+		else {
+			KNH_FREE(ctx, a->ilist, a->capacity * sizeof(knh_integer_t));
+		}
+		a->size = 0;
+		a->capacity = 0;
+	}
+}
+
+/* ======================================================================== */
+/* FArray */
+
+#define knh_FArray_init_ NULL
+#define knh_FArray_copy NULL
+#define knh_FArray_traverse_ NULL
+#define knh_FArray_compareTo NULL
+#define knh_FArray_hashCode NULL
+#define knh_FArray_newClass NULL
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_FArray_init(Ctx *ctx, knh_FArray_t *a, int init)
+{
+	a->flist = NULL;
+	a->capacity = 0;
+	a->size = 0;
+	if(init > 0) {
+		knh_FArray_grow(ctx, a, init, 0.0);
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_FArray_traverse(Ctx *ctx, knh_FArray_t *a, knh_ftraverse ftr)
+{
+	if(a->capacity > 0 && IS_SWEEP(ftr)) {
+		if(knh_FArray_isDim(a)) {
+			knh_array_dfree(ctx, a->flist, a->capacity, sizeof(knh_float_t));
+		}
+		else {
+			KNH_FREE(ctx, a->flist, a->capacity * sizeof(knh_float_t));
+		}
+		a->size = 0;
+		a->capacity = 0;
+	}
 }
 
 /* ======================================================================== */
@@ -1118,8 +1215,9 @@ void knh_Mapper_init(Ctx *ctx, Mapper *mpr, int init)
 	b->flag = 0;
 	b->scid = CLASS_Object;
 	b->tcid = CLASS_Object;
-	b->fmap = NULL;
+	//b->fmap = NULL;
 	KNH_INITv(b->mapdata, KNH_NULL);
+	KNH_INITv(b->m2, KNH_NULL);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -1128,13 +1226,8 @@ static
 void knh_Mapper_traverse(Ctx *ctx, Mapper *mpr, knh_ftraverse ftr)
 {
 	knh_Mapper_struct *b = DP(mpr);
-	if(IS_Mapper(b->mapdata)) {
-		ftr(ctx, UP(b->m1));
-		ftr(ctx, UP(b->m2));
-	}
-	else {
-		ftr(ctx, b->mapdata);
-	}
+	ftr(ctx, b->mapdata);
+	ftr(ctx, UP(b->m2));
 }
 
 /* ======================================================================== */
@@ -1735,7 +1828,7 @@ static
 void knh_ExceptionHandler_init(Ctx *ctx, ExceptionHandler *hdr, int init)
 {
 	knh_ExceptionHandler_struct *b = DP(hdr);
-	b->pc = NULL;
+	b->esp = ctx->esp;
 	KNH_INITv(b->caught, KNH_NULL);
 }
 
@@ -2031,7 +2124,7 @@ void knh_Stmt_done(Ctx *ctx, Stmt *o)
 {
 	SP(o)->stt = STT_DONE;
 	if(DP(o)->terms != NULL) {
-		knh_Stmt_terms_traverse(ctx, DP(o), knh_Object_RCsweep);
+		knh_Stmt_terms_traverse(ctx, DP(o), knh_Object_sweep);
 	}
 	KNH_SETv(ctx, DP(o)->metaDictMap, KNH_NULL);
 }
@@ -2195,6 +2288,8 @@ void knh_KLRCode_traverse(Ctx *ctx, KLRCode *o, knh_ftraverse ftr)
 #define knh_Tuple2_fdefault NULL
 #define knh_Range_fdefault NULL
 #define knh_Array_fdefault NULL
+#define knh_IArray_fdefault NULL
+#define knh_FArray_fdefault NULL
 #define knh_Iterator_fdefault NULL
 #define knh_DictMap_fdefault NULL
 #define knh_DictSet_fdefault NULL
