@@ -41,20 +41,20 @@ extern "C" {
 
 // stackf (Ctx *ctx, Object **sf)
 // this.func(a, b..)
-// ebp
+// esp
 // sf[-1]  sf[0]  sf[1]  sf[2]
 // sf[-1]  sf[0]  sf[1]  varg[0] varg[1]
 // method  this   a      b
 // method  this   a      b       b
-//                               ebp
+//                               esp
 // return
-// ebp
+// esp
 
 /* ------------------------------------------------------------------------ */
 
 int knh_sfp_argc(Ctx *ctx, knh_sfp_t *varg)
 {
-	return (((Context*)ctx)->ebp - varg);
+	return (((Context*)ctx)->esp - varg);
 }
 
 ///* ------------------------------------------------------------------------ */
@@ -63,7 +63,7 @@ int knh_sfp_argc(Ctx *ctx, knh_sfp_t *varg)
 //{
 //	Array *a = new_Array(ctx, CLASS_Any, knh_sfp_argc(ctx, varg));
 //	knh_sfp_t *p = varg;
-//	while(p <= ((Context*)ctx)->ebp) {
+//	while(p <= ((Context*)ctx)->esp) {
 //		knh_Array_add(ctx, a, p[0].o);
 //		p++;
 //	}
@@ -110,37 +110,41 @@ KNHAPI_(void) knh_sfp_unboxing(Ctx *ctx, knh_sfp_t *sfp)
 /* ======================================================================== */
 /* [call] */
 
-void knh_dynamiccall(Ctx *ctx, knh_sfp_t *mbp)
+char * knh_format_mtdparam(Ctx *ctx, char *buf, size_t bufsiz, Method *mtd, int n);
+
+
+void knh_sfp_typecheck(Ctx *ctx, knh_sfp_t *sfp, knh_type_t reqt, Method *mtd, knh_code_t *pc)
 {
-//	Method *mtd = mbp[0].mtd;
-//	knh_sfp_t *sfp = mbp + 1, *argv = mbp + 2;
-//	knh_code_t *pc = (knh_code_t*)sfp[-1].op;
-//	sfp[-1].op = 0;
-//
-//	KNH_ASSERT(IS_Method(mbp[0].mtd));
-//
-//	while(knh_sfp_argc(ctx, argv) < knh_Method_psize(mtd)) {
-//		KNH_LPUSH(ctx, KNH_NULL);
-//	}
-//
-//	{
-//		size_t i;
-//		for(i = 0; i < knh_Method_psize(mtd); i++) {
-//			knh_mfield_t f = knh_Method_pfields(mtd, i);
-//			knh_type_t t = knh_pmztype_totype(ctx, f.type, mbp[1].o->h.cid);
-//			VM_TYPECHK(ctx, t, argv[i].o);
-//		}
-//
-//		if(knh_Method_isVarArgs(mtd)) {
-//			knh_mfield_t f = knh_Method_lastfield(mtd);
-//			knh_type_t t = knh_pmztype_totype(ctx, f.type, mbp[1].o->h.cid);
-//			for(i = knh_Method_psize(mtd); i < knh_sfp_argc(ctx, argv); i++) {
-//				VM_TYPECHK(ctx, t, argv[i].o);
-//			}
-//		}
-//	}
-//	sfp[-1].op = (knh_stackop_t)pc;
-	TODO();
+	DBG2_ASSERT(sfp[-1].mtd == mtd);
+	if(reqt == TYPE_void && knh_Method_rztype(mtd) != TYPE_void) {
+		knh_throw__s(ctx, "Type!!: void", knh_Method_file(ctx, mtd), knh_Method_pctoline(mtd, pc));
+	}
+	char *emsg;
+	knh_class_t this_cid = knh_Object_cid(sfp[0].o);
+	int i, argc = knh_Method_isVarArgs(mtd) ? (ctx->esp - sfp) : knh_Method_psize(mtd);
+	for(i = 1; i < argc; i++) {
+		knh_type_t type = knh_Method_ptype(ctx, this_cid, mtd, i);
+		if(IS_NULL(sfp[i].o)) {
+			if(IS_NNTYPE(type)) {
+				emsg = "Null!!: "; goto L_THROWERR;
+			}
+		}
+		else {
+			knh_class_t reqc = CLASS_type(type);
+			if(!knh_class_instanceof(ctx, knh_Object_cid(sfp[i].o), reqc)) {
+				emsg = "Type!!: "; goto L_THROWERR;
+			}
+		}
+	}
+	return;
+
+	L_THROWERR:
+	{
+		char buf[CLASSNAME_BUFSIZ];
+		knh_snprintf(buf, sizeof(buf), emsg);
+		knh_format_mtdparam(ctx, buf+9, sizeof(buf)-9, mtd, i);
+		knh_throw__s(ctx, buf, knh_Method_file(ctx, mtd), knh_Method_pctoline(mtd, pc));
+	}
 }
 
 /* ======================================================================== */
@@ -153,13 +157,13 @@ void knh_sfp_dump(Ctx *ctx, knh_sfp_t *sfp)
 	if(sfp-1 > ctx->stack && IS_Method(sfp[-1].mtd)) {
 		char buf[CLASSNAME_BUFSIZ];
 		knh_format_cmethodn(ctx, buf, sizeof(buf), DP(sfp[-1].mtd)->cid, DP(sfp[-1].mtd)->mn);
-		fprintf(stderr, "sfp[-1] %s op=%d\n", buf, (int)sfp[-1].op);
+		fprintf(stderr, "sfp[-1] %s data=%d\n", buf, (int)sfp[-1].data);
 	}
-	for(i = 0; sfp + i < ctx->ebp; i++) {
-		fprintf(stderr, "sfp[%2d] %s op=%d\n", i, CLASSN(sfp[i].o->h.cid), (int)sfp[i].op);
+	for(i = 0; sfp + i < ctx->esp; i++) {
+		fprintf(stderr, "sfp[%2d] %s data=%d\n", i, CLASSN(sfp[i].o->h.cid), (int)sfp[i].data);
 	}
 	for(i = 0; i < 2; i++) {
-		fprintf(stderr, "ebp[%2d] %s op=%d\n", i, CLASSN(ctx->ebp[i].o->h.cid), (int)sfp[i].op);
+		fprintf(stderr, "esp[%2d] %s data=%d\n", i, CLASSN(ctx->esp[i].o->h.cid), (int)sfp[i].data);
 	}
 	fprintf(stderr, "** ** ** ** ** ** ** ** **\n");
 }
@@ -208,9 +212,9 @@ KNHAPI(void) knh_sfp_format(Ctx *ctx, knh_sfp_t *sfp, knh_methodn_t mn, OutputSt
 	knh_throwException(ctx, new_Exception__b(ctx, B(s)), KNH_SAFEFILE(__FILE__), __LINE__); \
 
 #define _KNH_THROWf(ctx, fmt, ...) \
-	char throwbuf_[256]; \
-	knh_snprintf(throwbuf_, sizeof(throwbuf_), fmt, ## __VA_ARGS__); \
-	knh_throwException(ctx, new_Exception__b(ctx, B(throwbuf_)), KNH_SAFEFILE(__FILE__), __LINE__); \
+	char throcwb_[256]; \
+	knh_snprintf(throcwb_, sizeof(throcwb_), fmt, ## __VA_ARGS__); \
+	knh_throwException(ctx, new_Exception__b(ctx, B(throcwb_)), KNH_SAFEFILE(__FILE__), __LINE__); \
 
 #define DEBUG_THROWf(ctx, fmt, ...) \
 	fprintf(stderr, "THROW?[%s:%d]: ", __FUNCTION__, __LINE__); \
@@ -249,66 +253,41 @@ void knh_Exception_addStackTrace(Ctx *ctx, Exception *o, String *msg)
 	}
 }
 
-/* ------------------------------------------------------------------------ */
-
-static
-String *knh_stackf_getStackTraceMsg(Ctx *ctx, knh_sfp_t *sfp)
-{
-	char *fn = "-";
-	int  line = 0;
-	if(sfp[-1].op != 0) {
-		knh_sfp_t *prev_sfp = sfp;
-		while(ctx->stack <= prev_sfp) {
-			if(IS_Method(prev_sfp[0].o) && prev_sfp[0].op != 0) {
-				fn = knh_Method_file(ctx, prev_sfp[0].mtd);
-				line = knh_Method_pctoline(prev_sfp[0].mtd, (knh_code_t*)sfp[-1].op);
-				break;
-			}
-			prev_sfp--;
-		}
-	}
-	char bufm[CLASSNAME_BUFSIZ];
-	char bufn[CLASSNAME_BUFSIZ*2];
-	knh_snprintf(bufn, sizeof(bufn), "%s.%s:(%s:%d)",
-		CLASSN(knh_Object_cid(sfp[0].o)), knh_format_methodn(ctx, bufm, sizeof(bufm), DP(sfp[-1].mtd)->mn), fn, line);
-	return new_String(ctx, B(bufn), NULL);
-}
-
-/* ------------------------------------------------------------------------ */
-/* @method void Context.%s(OutputStream w, String m) */
-
-void knh_Context__s(Ctx *ctx, Context *o, OutputStream *w, String *m)
-{
-	char *fn = NULL;
-	int  line = 0;
-	knh_sfp_t *mtd_sfp = NULL, *prev_sfp = o->ebp;
-	while(o->stack <= prev_sfp) {
-		if(IS_Method(prev_sfp[0].o) && prev_sfp[0].op != 0) {
-			if(mtd_sfp != NULL) {
-				fn = knh_Method_file(ctx, prev_sfp[0].mtd);
-				line = knh_Method_pctoline(prev_sfp[0].mtd, (knh_code_t*)mtd_sfp[-1].op);
-				break;
-			}
-			mtd_sfp = prev_sfp;
-		}
-		prev_sfp--;
-	}
-	knh_putc(ctx, w, '[');
-	if(o->doing == NULL) {
-		knh_write__s(ctx, w, KONOHA_NAME);
-	}
-	else {
-		knh_write__s(ctx, w, o->doing);
-	}
-	knh_putc(ctx, w, ']');
-	if(fn != NULL) {
-		knh_putc(ctx, w, '(');
-		knh_write__s(ctx, w, KNH_SAFEFILE(fn));
-		knh_putc(ctx, w, ':');
-		knh_write__i(ctx, w, (knh_int_t)line);
-		knh_putc(ctx, w, ')');
-	}
-}
+///* ------------------------------------------------------------------------ */
+///* @method void Context.%s(OutputStream w, String m) */
+//
+//void knh_Context__s(Ctx *ctx, Context *o, OutputStream *w, String *m)
+//{
+//	char *fn = NULL;
+//	int  line = 0;
+//	knh_sfp_t *mtd_sfp = NULL, *prev_sfp = o->esp;
+//	while(o->stack <= prev_sfp) {
+//		if(IS_Method(prev_sfp[0].o) && prev_sfp[0].op != 0) {
+//			if(mtd_sfp != NULL) {
+//				fn = knh_Method_file(ctx, prev_sfp[0].mtd);
+//				line = knh_Method_pctoline(prev_sfp[0].mtd, (knh_code_t*)mtd_sfp[-1].op);
+//				break;
+//			}
+//			mtd_sfp = prev_sfp;
+//		}
+//		prev_sfp--;
+//	}
+//	knh_putc(ctx, w, '[');
+//	if(o->doing == NULL) {
+//		knh_write__s(ctx, w, KONOHA_NAME);
+//	}
+//	else {
+//		knh_write__s(ctx, w, o->doing);
+//	}
+//	knh_putc(ctx, w, ']');
+//	if(fn != NULL) {
+//		knh_putc(ctx, w, '(');
+//		knh_write__s(ctx, w, KNH_SAFEFILE(fn));
+//		knh_putc(ctx, w, ':');
+//		knh_write__i(ctx, w, (knh_int_t)line);
+//		knh_putc(ctx, w, ')');
+//	}
+//}
 
 /* ------------------------------------------------------------------------ */
 
@@ -321,34 +300,50 @@ KNHAPI(char*) konoha_doing(Ctx *ctx, char *msg)
 
 /* ------------------------------------------------------------------------ */
 
+
+
+/* ------------------------------------------------------------------------ */
+
+static
+String *knh_stackf_getStackTraceMsg(Ctx *ctx, knh_sfp_t *sfp)
+{
+	Method *mtd = sfp[-1].mtd;
+	knh_code_t *pc = sfp[-1].pc;
+	DBG2_P("pc=%p, mtd->func=%p, mtd->pc=%p", pc, mtd->fcall_1, mtd->pc_start);
+	char *fn = "-";
+	int  line = 0;
+	if(pc != NULL && pc != (knh_code_t*)mtd->fcall_1 && pc != mtd->pc_start) {
+		fn = knh_Method_file(ctx, mtd);
+		line = knh_Method_pctoline(mtd, pc);
+	}
+	knh_cwb_t cwb = new_cwb(ctx);
+	knh_write_cidmn(ctx, cwb.w, DP(mtd)->cid, DP(mtd)->mn);
+	knh_putc(ctx, cwb.w, ':');
+	knh_write_fline(ctx, cwb.w, fn, line);
+	return new_String__cwb(ctx, cwb);
+}
+
+/* ------------------------------------------------------------------------ */
+
 KNHAPI(void) knh_throwException(Ctx *ctx, Exception *e, char *file, int line)
 {
-	Context *o = (Context*)ctx;
 	KNH_ASSERT(IS_Exception(e));
 	if(file != NULL) {
 		DP(e)->file = file;
 		DP(e)->line = line;
 	}
-	if(ctx == NULL) {
-		goto L_ERROR;
-	}
-	((Context*)ctx)->doing = NULL; /* reset */
-	while(o->stack <= o->ebp) {
-		//DBG2_P("stack[%d]\t %s\n", (o->ebp - o->stack), CLASSNo(o->ebp[0].o));
-		if(IS_ExceptionHandler(o->ebp[0].hdr) && knh_ExceptionHandler_isCatching(o->ebp[0].hdr)) {
-			knh_ExceptionHandler_setCatching(o->ebp[0].hdr, 0);
-			knh_ExceptionHandler_longjmp(ctx, o->ebp[0].hdr, e);
+	knh_sfp_t *sp = ctx->esp;
+	while(ctx->stack <= sp) {
+		if(IS_ExceptionHandler(sp[0].hdr) && knh_ExceptionHandler_isCatching(sp[0].hdr)) {
+			knh_ExceptionHandler_setCatching(sp[0].hdr, 0);
+			knh_ExceptionHandler_longjmp(ctx, sp[0].hdr, e);
 		}
-		else if(IS_Method(o->ebp[0].o)) {
-			//DBG2_P("pc = %p", (void*)o->ebp[0].op);
-			if(o->ebp[0].op != 0) {
-				knh_Exception_addStackTrace(ctx, e, knh_stackf_getStackTraceMsg(ctx, o->ebp+1));
-			}
+		else if(IS_Method(sp[0].o)) {
+			knh_Exception_addStackTrace(ctx, e, knh_stackf_getStackTraceMsg(ctx, sp+1));
 		}
-		o->ebp--;
+		sp--;
 	}
 
-	L_ERROR:;
 	fprintf(stderr, "********** USE STACKTRACE IN YOUR C/C++ DEBUGGER ************\n");
 	fprintf(stderr, "Uncaught Exception: %s\n", knh_String_tochar(DP(e)->message));
 	fprintf(stderr, "*************************************************************\n");
@@ -357,6 +352,13 @@ KNHAPI(void) knh_throwException(Ctx *ctx, Exception *e, char *file, int line)
 
 /* ------------------------------------------------------------------------ */
 
+KNHAPI(void) knh_throw__s(Ctx *ctx, char *msg, char *file, int line)
+{
+	knh_throwException(ctx, new_Exception__b(ctx, B(msg)), file, line);
+}
+
+/* ------------------------------------------------------------------------ */
+
 KNHAPI(void) knh_throw(Ctx *ctx, Object *e, char *file, int line)
 {
 	if(IS_NULL(e)) {
